@@ -42,7 +42,7 @@ class VolumeTypeExtraSpecTemplate(xmlutil.TemplateBuilder):
 
         def extraspec_sel(obj, do_raise=False):
             # Have to extract the key and value for later use...
-            key, value = obj.items()[0]
+            key, value = list(obj.items())[0]
             return dict(key=key, value=value)
 
         root = xmlutil.TemplateElement(tagname, selector=extraspec_sel)
@@ -56,7 +56,7 @@ class VolumeTypeExtraSpecsController(wsgi.Controller):
     def _get_extra_specs(self, context, type_id):
         extra_specs = db.volume_type_extra_specs_get(context, type_id)
         specs_dict = {}
-        for key, value in extra_specs.iteritems():
+        for key, value in extra_specs.items():
             specs_dict[key] = value
         return dict(extra_specs=specs_dict)
 
