@@ -15,6 +15,7 @@
 
 from oslo_log import log as logging
 import oslo_messaging as messaging
+from oslo_utils import encodeutils
 from oslo_utils import strutils
 import six
 import webob
@@ -98,15 +99,7 @@ class VolumeActionsController(wsgi.Controller):
         else:
             mode = 'rw'
 
-        if instance_uuid and host_name:
-            msg = _("Invalid request to attach volume to an "
-                    "instance %(instance_uuid)s and a "
-                    "host %(host_name)s simultaneously") % {
-                'instance_uuid': instance_uuid,
-                'host_name': host_name,
-            }
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-        elif instance_uuid is None and host_name is None:
+        if instance_uuid is None and host_name is None:
             msg = _("Invalid request to attach volume to an invalid target")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
@@ -269,7 +262,8 @@ class VolumeActionsController(wsgi.Controller):
         try:
             force = strutils.bool_from_string(force, strict=True)
         except ValueError as error:
-            msg = _("Invalid value for 'force': '%s'") % error.message
+            err_msg = encodeutils.exception_to_unicode(error)
+            msg = _("Invalid value for 'force': '%s'") % err_msg
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         try:
@@ -315,7 +309,11 @@ class VolumeActionsController(wsgi.Controller):
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         size = int(body['os-extend']['new_size'])
-        self.volume_api.extend(context, volume, size)
+        try:
+            self.volume_api.extend(context, volume, size)
+        except exception.InvalidVolume as error:
+            raise webob.exc.HTTPBadRequest(explanation=error.msg)
+
         return webob.Response(status_int=202)
 
     @wsgi.action('os-update_readonly_flag')
@@ -337,7 +335,8 @@ class VolumeActionsController(wsgi.Controller):
             readonly_flag = strutils.bool_from_string(readonly_flag,
                                                       strict=True)
         except ValueError as error:
-            msg = _("Invalid value for 'readonly': '%s'") % error.message
+            err_msg = encodeutils.exception_to_unicode(error)
+            msg = _("Invalid value for 'readonly': '%s'") % err_msg
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         self.volume_api.update_readonly_flag(context, volume, readonly_flag)
@@ -377,7 +376,8 @@ class VolumeActionsController(wsgi.Controller):
             bootable = strutils.bool_from_string(bootable,
                                                  strict=True)
         except ValueError as error:
-            msg = _("Invalid value for 'bootable': '%s'") % error.message
+            err_msg = encodeutils.exception_to_unicode(error)
+            msg = _("Invalid value for 'bootable': '%s'") % err_msg
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         update_dict = {'bootable': bootable}
