@@ -13,10 +13,11 @@
 #   under the License.
 
 """The Volume Image Metadata API extension."""
-import logging
-
+import six
 import webob
 
+from oslo_log import log as logging
+
 from cinder.api import common
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
@@ -46,6 +47,16 @@ class VolumeImageMetadataController(wsgi.Controller):
             raise webob.exc.HTTPNotFound(explanation=msg)
         return (volume, meta)
 
+    def _get_all_images_metadata(self, context):
+        """Returns the image metadata for all volumes."""
+        try:
+            all_metadata = self.volume_api.get_volumes_image_metadata(context)
+        except Exception as e:
+            LOG.debug('Problem retrieving volume image metadata. '
+                      'It will be skipped. Error: %s', six.text_type(e))
+            all_metadata = {}
+        return all_metadata
+
     def _add_image_metadata(self, context, resp_volume_list, image_metas=None):
         """Appends the image metadata to each of the given volume.
 
@@ -84,9 +95,8 @@ class VolumeImageMetadataController(wsgi.Controller):
         if authorize(context):
             resp_obj.attach(xml=VolumesImageMetadataTemplate())
             # Just get the image metadata of those volumes in response.
-            volumes = list(resp_obj.obj.get('volumes', []))
-            if volumes:
-                self._add_image_metadata(context, volumes)
+            self._add_image_metadata(context,
+                                     list(resp_obj.obj.get('volumes', [])))
 
     @wsgi.action("os-set_image_metadata")
     @wsgi.serializers(xml=common.MetadataTemplate)
@@ -129,6 +139,12 @@ class VolumeImageMetadataController(wsgi.Controller):
         except exception.InvalidVolumeMetadataSize as error:
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=error.msg)
 
+    @wsgi.action("os-show_image_metadata")
+    @wsgi.serializers(xml=common.MetadataTemplate)
+    def index(self, req, id, body):
+        context = req.environ['cinder.context']
+        return {'metadata': self._get_image_metadata(context, id)[1]}
+
     @wsgi.action("os-unset_image_metadata")
     def delete(self, req, id, body):
         """Deletes an existing image metadata."""
