@@ -16,8 +16,6 @@
 """The volumes api."""
 
 
-import ast
-
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
@@ -37,18 +35,7 @@ from cinder import volume as cinder_volume
 from cinder.volume import utils as volume_utils
 from cinder.volume import volume_types
 
-
-query_volume_filters_opt = cfg.ListOpt('query_volume_filters',
-                                       default=['name', 'status', 'metadata',
-                                                'availability_zone'],
-                                       help="Volume filter options which "
-                                            "non-admin user could use to "
-                                            "query volumes. Default values "
-                                            "are: ['name', 'status', "
-                                            "'metadata', 'availability_zone']")
-
 CONF = cfg.CONF
-CONF.register_opt(query_volume_filters_opt)
 
 LOG = logging.getLogger(__name__)
 SCHEDULER_HINTS_NAMESPACE =\
@@ -199,16 +186,15 @@ class VolumeController(wsgi.Controller):
         """Delete a volume."""
         context = req.environ['cinder.context']
 
+        cascade = utils.get_bool_param('cascade', req.params)
+
         LOG.info(_LI("Delete volume with id: %s"), id, context=context)
 
         try:
             volume = self.volume_api.get(context, id)
-            self.volume_api.delete(context, volume)
+            self.volume_api.delete(context, volume, cascade=cascade)
         except exception.VolumeNotFound as error:
             raise exc.HTTPNotFound(explanation=error.msg)
-        except exception.VolumeAttached:
-            msg = _("Volume cannot be deleted while in attached state")
-            raise exc.HTTPBadRequest(explanation=msg)
         return webob.Response(status_int=202)
 
     @wsgi.serializers(xml=VolumesTemplate)
@@ -243,12 +229,6 @@ class VolumeController(wsgi.Controller):
             filters['display_name'] = filters['name']
             del filters['name']
 
-        for k, v in filters.items():
-            try:
-                filters[k] = ast.literal_eval(v)
-            except (ValueError, SyntaxError):
-                LOG.debug('Could not evaluate value %s, assuming string', v)
-
         self.volume_api.check_volume_filters(filters)
         volumes = self.volume_api.get_all(context, marker, limit,
                                           sort_keys=sort_keys,
@@ -257,12 +237,10 @@ class VolumeController(wsgi.Controller):
                                           viewable_admin_meta=True,
                                           offset=offset)
 
-        volumes = [dict(vol) for vol in volumes]
-
         for volume in volumes:
             utils.add_visible_admin_metadata(volume)
 
-        req.cache_db_volumes(volumes)
+        req.cache_db_volumes(volumes.objects)
 
         if is_detail:
             volumes = self._view_builder.detail_list(req, volumes)
@@ -426,10 +404,6 @@ class VolumeController(wsgi.Controller):
                                             volume.get('display_description'),
                                             **kwargs)
 
-        # TODO(vish): Instance should be None at db layer instead of
-        #             trying to lazy load, but for now we turn it into
-        #             a dict to avoid an error.
-        new_volume = dict(new_volume)
         retval = self._view_builder.detail(req, new_volume)
 
         return retval
