@@ -20,51 +20,59 @@ Client side of the volume backup RPC API.
 
 from oslo_config import cfg
 from oslo_log import log as logging
-import oslo_messaging as messaging
 
-from cinder.objects import base as objects_base
 from cinder import rpc
 
 
 CONF = cfg.CONF
-
 LOG = logging.getLogger(__name__)
 
 
-class BackupAPI(object):
+class BackupAPI(rpc.RPCAPI):
     """Client side of the volume rpc API.
 
     API version history:
 
         1.0 - Initial version.
         1.1 - Changed methods to accept backup objects instead of IDs.
+        1.2 - A version that got in by mistake (without breaking anything).
+        1.3 - Dummy version bump to mark start of having cinder-backup service
+              decoupled from cinder-volume.
+
+        ... Mitaka supports messaging 1.3. Any changes to existing methods in
+        1.x after this point should be done so that they can handle version cap
+        set to 1.3.
+
+        2.0 - Remove 1.x compatibility
     """
 
-    BASE_RPC_API_VERSION = '1.0'
-    RPC_API_VERSION = '1.1'
+    RPC_API_VERSION = '1.3'
+    TOPIC = CONF.backup_topic
+    BINARY = 'cinder-backup'
 
-    def __init__(self):
-        super(BackupAPI, self).__init__()
-        target = messaging.Target(topic=CONF.backup_topic,
-                                  version=self.BASE_RPC_API_VERSION)
-        serializer = objects_base.CinderObjectSerializer()
-        self.client = rpc.get_client(target, self.RPC_API_VERSION,
-                                     serializer=serializer)
+    def _compat_ver(self, current, legacy):
+        if self.client.can_send_version(current):
+            return current
+        else:
+            return legacy
 
     def create_backup(self, ctxt, backup):
         LOG.debug("create_backup in rpcapi backup_id %s", backup.id)
-        cctxt = self.client.prepare(server=backup.host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=backup.host, version=version)
         cctxt.cast(ctxt, 'create_backup', backup=backup)
 
     def restore_backup(self, ctxt, volume_host, backup, volume_id):
         LOG.debug("restore_backup in rpcapi backup_id %s", backup.id)
-        cctxt = self.client.prepare(server=volume_host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=volume_host, version=version)
         cctxt.cast(ctxt, 'restore_backup', backup=backup,
                    volume_id=volume_id)
 
     def delete_backup(self, ctxt, backup):
         LOG.debug("delete_backup  rpcapi backup_id %s", backup.id)
-        cctxt = self.client.prepare(server=backup.host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=backup.host, version=version)
         cctxt.cast(ctxt, 'delete_backup', backup=backup)
 
     def export_record(self, ctxt, backup):
@@ -72,7 +80,8 @@ class BackupAPI(object):
                   "on host %(host)s.",
                   {'id': backup.id,
                    'host': backup.host})
-        cctxt = self.client.prepare(server=backup.host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=backup.host, version=version)
         return cctxt.call(ctxt, 'export_record', backup=backup)
 
     def import_record(self,
@@ -87,7 +96,8 @@ class BackupAPI(object):
                   {'id': backup.id,
                    'host': host,
                    'url': backup_url})
-        cctxt = self.client.prepare(server=host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=host, version=version)
         cctxt.cast(ctxt, 'import_record',
                    backup=backup,
                    backup_service=backup_service,
@@ -99,11 +109,13 @@ class BackupAPI(object):
                   "on host %(host)s.",
                   {'id': backup.id,
                    'host': backup.host})
-        cctxt = self.client.prepare(server=backup.host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=backup.host, version=version)
         return cctxt.cast(ctxt, 'reset_status', backup=backup, status=status)
 
     def check_support_to_force_delete(self, ctxt, host):
         LOG.debug("Check if backup driver supports force delete "
                   "on host %(host)s.", {'host': host})
-        cctxt = self.client.prepare(server=host)
+        version = self._compat_ver('2.0', '1.1')
+        cctxt = self.client.prepare(server=host, version=version)
         return cctxt.call(ctxt, 'check_support_to_force_delete')
