@@ -17,12 +17,14 @@
 
 """Starter script for Cinder Volume Backup."""
 
+import logging as python_logging
 import sys
 
 import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_reports import guru_meditation_report as gmr
+from oslo_reports import opts as gmr_opts
 
 eventlet.monkey_patch()
 
@@ -42,11 +44,13 @@ CONF = cfg.CONF
 
 def main():
     objects.register_all()
+    gmr_opts.set_defaults(CONF)
     CONF(sys.argv[1:], project='cinder',
          version=version.version_string())
     logging.setup(CONF, "cinder")
+    python_logging.captureWarnings(True)
     utils.monkey_patch()
-    gmr.TextGuruMeditation.setup_autorun(version)
+    gmr.TextGuruMeditation.setup_autorun(version, conf=CONF)
     server = service.Service.create(binary='cinder-backup')
     service.serve(server)
     service.wait()
