@@ -34,12 +34,12 @@ from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import fileutils
+from oslo_utils import imageutils
 from oslo_utils import timeutils
 from oslo_utils import units
 
 from cinder import exception
-from cinder.i18n import _, _LE, _LI, _LW
-from cinder.openstack.common import imageutils
+from cinder.i18n import _, _LI, _LW
 from cinder import utils
 from cinder.volume import throttling
 from cinder.volume import utils as volume_utils
@@ -54,31 +54,13 @@ image_helper_opts = [cfg.StrOpt('image_conversion_dir',
 CONF = cfg.CONF
 CONF.register_opts(image_helper_opts)
 
-QEMU_IMG_LIMITS = None
-
-try:
-    QEMU_IMG_LIMITS = processutils.ProcessLimits(
-        cpu_time=2,
-        address_space=1 * units.Gi)
-except Exception:
-    LOG.error(_LE('Use of process limits requires oslo.concurrency '
-                  'version >=2.6.1 but the installed version is '
-                  'older than that.  Please upgrade oslo.concurrency '
-                  'to address vulnerability CVE-2015-5162.'))
-
 
 def qemu_img_info(path, run_as_root=True):
-    """Return a object containing the parsed output from qemu-img info."""
+    """Return an object containing the parsed output from qemu-img info."""
     cmd = ('env', 'LC_ALL=C', 'qemu-img', 'info', path)
     if os.name == 'nt':
         cmd = cmd[2:]
-
-    if QEMU_IMG_LIMITS:
-        out, _err = utils.execute(*cmd, run_as_root=run_as_root,
-                                  prlimit=QEMU_IMG_LIMITS)
-    else:
-        out, _err = utils.execute(*cmd, run_as_root=run_as_root)
-
+    out, _err = utils.execute(*cmd, run_as_root=run_as_root)
     return imageutils.QemuImgInfo(out)
 
 
@@ -98,7 +80,8 @@ def _get_version_from_string(version_string):
 
 def check_qemu_img_version(minimum_version):
     qemu_version = get_qemu_img_version()
-    if qemu_version < _get_version_from_string(minimum_version):
+    if (qemu_version is None
+       or qemu_version < _get_version_from_string(minimum_version)):
         if qemu_version:
             current_version = '.'.join((str(element)
                                        for element in qemu_version))
@@ -143,7 +126,17 @@ def _convert_image(prefix, source, dest, out_format, run_as_root=True):
     # some incredible event this is 0 (cirros image?) don't barf
     if duration < 1:
         duration = 1
-    fsz_mb = os.stat(source).st_size / units.Mi
+    try:
+        image_size = qemu_img_info(source, run_as_root=True).virtual_size
+    except ValueError as e:
+        msg = _LI("The image was successfully converted, but image size "
+                  "is unavailable. src %(src)s, dest %(dest)s. %(error)s")
+        LOG.info(msg, {"src": source,
+                       "dest": dest,
+                       "error": e})
+        return
+
+    fsz_mb = image_size / units.Mi
     mbps = (fsz_mb / duration)
     msg = ("Image conversion details: src %(src)s, size %(sz).2f MB, "
            "duration %(duration).2f sec, destination %(dest)s")
