@@ -21,6 +21,7 @@ inline callbacks.
 
 """
 
+import copy
 import logging
 import os
 import shutil
@@ -43,20 +44,15 @@ from cinder.common import config  # noqa Need to register global_opts
 from cinder.db import migration
 from cinder.db.sqlalchemy import api as sqla_api
 from cinder import i18n
-from cinder import objects
+from cinder.objects import base as objects_base
 from cinder import rpc
 from cinder import service
 from cinder.tests import fixtures as cinder_fixtures
 from cinder.tests.unit import conf_fixture
 from cinder.tests.unit import fake_notifier
 
-test_opts = [
-    cfg.StrOpt('sqlite_clean_db',
-               default='clean.sqlite',
-               help='File name of clean sqlite db'), ]
 
 CONF = cfg.CONF
-CONF.register_opts(test_opts)
 
 LOG = log.getLogger(__name__)
 
@@ -105,6 +101,29 @@ class Database(fixtures.Fixture):
                 os.path.join(CONF.state_path, self.sqlite_db))
 
 
+def _patch_mock_to_raise_for_invalid_assert_calls():
+    def raise_for_invalid_assert_calls(wrapped):
+        def wrapper(_self, name):
+            valid_asserts = [
+                'assert_called_with',
+                'assert_called_once_with',
+                'assert_has_calls',
+                'assert_any_call']
+
+            if name.startswith('assert') and name not in valid_asserts:
+                raise AttributeError('%s is not a valid mock assert method'
+                                     % name)
+
+            return wrapped(_self, name)
+        return wrapper
+    mock.Mock.__getattr__ = raise_for_invalid_assert_calls(
+        mock.Mock.__getattr__)
+
+# NOTE(gibi): needs to be called only once at import time
+# to patch the mock lib
+_patch_mock_to_raise_for_invalid_assert_calls()
+
+
 class TestCase(testtools.TestCase):
     """Test case base class for all unit tests."""
 
@@ -128,9 +147,6 @@ class TestCase(testtools.TestCase):
                        side_effect=self._get_joined_notifier)
         p.start()
 
-        # Import cinder objects for test cases
-        objects.register_all()
-
         # Unit tests do not need to use lazy gettext
         i18n.enable_lazy(False)
 
@@ -167,6 +183,14 @@ class TestCase(testtools.TestCase):
         self.useFixture(self.messaging_conf)
         rpc.init(CONF)
 
+        # NOTE(geguileo): This is required because _determine_obj_version_cap
+        # and _determine_rpc_version_cap functions in cinder.rpc.RPCAPI cache
+        # versions in LAST_RPC_VERSIONS and LAST_OBJ_VERSIONS so we may have
+        # weird interactions between tests if we don't clear them before each
+        # test.
+        rpc.LAST_OBJ_VERSIONS = {}
+        rpc.LAST_RPC_VERSIONS = {}
+
         conf_fixture.set_defaults(CONF)
         CONF([], default_config_files=[])
 
@@ -183,9 +207,17 @@ class TestCase(testtools.TestCase):
             _DB_CACHE = Database(sqla_api, migration,
                                  sql_connection=CONF.database.connection,
                                  sqlite_db=CONF.database.sqlite_db,
-                                 sqlite_clean_db=CONF.sqlite_clean_db)
+                                 sqlite_clean_db='clean.sqlite')
         self.useFixture(_DB_CACHE)
 
+        # NOTE(danms): Make sure to reset us back to non-remote objects
+        # for each test to avoid interactions. Also, backup the object
+        # registry.
+        objects_base.CinderObject.indirection_api = None
+        self._base_test_obj_backup = copy.copy(
+            objects_base.CinderObjectRegistry._registry._obj_classes)
+        self.addCleanup(self._restore_obj_registry)
+
         # emulate some of the mox stuff, we can't use the metaclass
         # because it screws with our generators
         mox_fixture = self.useFixture(moxstubout.MoxStubout())
@@ -219,6 +251,16 @@ class TestCase(testtools.TestCase):
 
         self._disable_osprofiler()
 
+        # NOTE(geguileo): This is required because common get_by_id method in
+        # cinder.db.sqlalchemy.api caches get methods and if we use a mocked
+        # get method in one test it would carry on to the next test.  So we
+        # clear out the cache.
+        sqla_api._GET_METHODS = {}
+
+    def _restore_obj_registry(self):
+        objects_base.CinderObjectRegistry._registry._obj_classes = \
+            self._base_test_obj_backup
+
     def _disable_osprofiler(self):
         """Disable osprofiler.
 
