@@ -15,8 +15,11 @@
 
 import mock
 
+import oslo_messaging
+
 from cinder.api.contrib import capabilities
 from cinder import context
+from cinder import exception
 from cinder import test
 from cinder.tests.unit.api import fakes
 
@@ -31,6 +34,7 @@ def rpcapi_get_capabilities(self, context, host, discover):
         display_name='Capabilities of Cinder LVM driver',
         description='These are volume type options provided by '
                     'Cinder LVM driver, blah, blah.',
+        replication_targets=[],
         visibility='public',
         properties = dict(
             compression = dict(
@@ -54,8 +58,6 @@ def rpcapi_get_capabilities(self, context, host, discover):
     return capabilities
 
 
-@mock.patch('cinder.volume.rpcapi.VolumeAPI.get_capabilities',
-            rpcapi_get_capabilities)
 class CapabilitiesAPITest(test.TestCase):
     def setUp(self):
         super(CapabilitiesAPITest, self).setUp()
@@ -63,7 +65,11 @@ class CapabilitiesAPITest(test.TestCase):
         self.controller = capabilities.CapabilitiesController()
         self.ctxt = context.RequestContext('admin', 'fake', True)
 
-    def test_capabilities_summary(self):
+    @mock.patch('cinder.db.service_get_all')
+    @mock.patch('cinder.volume.rpcapi.VolumeAPI.get_capabilities',
+                rpcapi_get_capabilities)
+    def test_capabilities_summary(self, mock_services):
+        mock_services.return_value = [{'name': 'fake'}]
         req = fakes.HTTPRequest.blank('/fake/capabilities/fake')
         req.environ['cinder.context'] = self.ctxt
         res = self.controller.show(req, 'fake')
@@ -79,6 +85,7 @@ class CapabilitiesAPITest(test.TestCase):
             'description': 'These are volume type options provided by '
                            'Cinder LVM driver, blah, blah.',
             'visibility': 'public',
+            'replication_targets': [],
             'properties': {
                 'compression': {
                     'title': 'Compression',
@@ -100,3 +107,23 @@ class CapabilitiesAPITest(test.TestCase):
         }
 
         self.assertDictMatch(expected, res)
+
+    @mock.patch('cinder.db.service_get_all')
+    @mock.patch('cinder.volume.rpcapi.VolumeAPI.get_capabilities')
+    def test_get_capabilities_rpc_timeout(self, mock_rpc, mock_services):
+        mock_rpc.side_effect = oslo_messaging.MessagingTimeout
+        mock_services.return_value = [{'name': 'fake'}]
+
+        req = fakes.HTTPRequest.blank('/fake/capabilities/fake')
+        req.environ['cinder.context'] = self.ctxt
+        self.assertRaises(exception.RPCTimeout,
+                          self.controller.show, req, 'fake')
+
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_capabilities_service_not_found(self, mock_services):
+        mock_services.return_value = []
+
+        req = fakes.HTTPRequest.blank('/fake/capabilities/fake')
+        req.environ['cinder.context'] = self.ctxt
+        self.assertRaises(exception.NotFound,
+                          self.controller.show, req, 'fake')
