@@ -13,27 +13,28 @@
 #   under the License.
 
 import datetime
-import json
+import iso8601
 import uuid
 
 import mock
-from oslo_config import cfg
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
 import webob
 
 from cinder.api.contrib import volume_actions
+from cinder import context
 from cinder import exception
 from cinder.image import glance
+from cinder import objects
 from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import stubs
+from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import fake_volume
 from cinder import volume
 from cinder.volume import api as volume_api
 from cinder.volume import rpcapi as volume_rpcapi
 
-CONF = cfg.CONF
-
 
 class VolumeActionsTest(test.TestCase):
 
@@ -43,6 +44,7 @@ class VolumeActionsTest(test.TestCase):
 
     def setUp(self):
         super(VolumeActionsTest, self).setUp()
+        self.context = context.RequestContext('fake', 'fake', is_admin=False)
         self.UUID = uuid.uuid4()
         self.controller = volume_actions.VolumeActionsController()
         self.api_patchers = {}
@@ -53,8 +55,10 @@ class VolumeActionsTest(test.TestCase):
             self.addCleanup(self.api_patchers[_meth].stop)
             self.api_patchers[_meth].return_value = True
 
-        vol = {'id': 'fake', 'host': 'fake', 'status': 'available', 'size': 1,
-               'migration_status': None, 'volume_type_id': 'fake'}
+        db_vol = {'id': 'fake', 'host': 'fake', 'status': 'available',
+                  'size': 1, 'migration_status': None,
+                  'volume_type_id': 'fake', 'project_id': 'project_id'}
+        vol = fake_volume.fake_volume_obj(self.context, **db_vol)
         self.get_patcher = mock.patch('cinder.volume.api.API.get')
         self.mock_volume_get = self.get_patcher.start()
         self.addCleanup(self.get_patcher.stop)
@@ -63,6 +67,10 @@ class VolumeActionsTest(test.TestCase):
         self.mock_volume_update = self.update_patcher.start()
         self.addCleanup(self.update_patcher.stop)
         self.mock_volume_update.return_value = vol
+        self.db_get_patcher = mock.patch('cinder.db.sqlalchemy.api.volume_get')
+        self.mock_volume_db_get = self.db_get_patcher.start()
+        self.addCleanup(self.db_get_patcher.stop)
+        self.mock_volume_db_get.return_value = vol
 
         self.flags(rpc_backend='cinder.openstack.common.rpc.impl_fake')
 
@@ -72,7 +80,7 @@ class VolumeActionsTest(test.TestCase):
             req = webob.Request.blank('/v2/fake/volumes/%s/action' %
                                       self.UUID)
             req.method = 'POST'
-            req.body = jsonutils.dumps({_action: None})
+            req.body = jsonutils.dump_as_bytes({_action: None})
             req.content_type = 'application/json'
             res = req.get_response(app)
             self.assertEqual(202, res.status_int)
@@ -84,7 +92,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-initialize_connection': {'connector': 'fake'}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -97,7 +105,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-initialize_connection': {}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -111,7 +119,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-initialize_connection': {'connector': 'fake'}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -124,7 +132,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-terminate_connection': {'connector': 'fake'}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -137,7 +145,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-terminate_connection': {}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -151,7 +159,7 @@ class VolumeActionsTest(test.TestCase):
             body = {'os-terminate_connection': {'connector': 'fake'}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
 
             res = req.get_response(fakes.wsgi_app())
@@ -163,19 +171,29 @@ class VolumeActionsTest(test.TestCase):
                               'mode': 'rw'}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(202, res.status_int)
 
+        body = {'os-attach': {'instance_uuid': 'fake',
+                              'host_name': 'fake_host',
+                              'mountpoint': '/dev/vdc'}}
+        req = webob.Request.blank('/v2/fake/volumes/1/action')
+        req.method = "POST"
+        req.headers["content-type"] = "application/json"
+        req.body = jsonutils.dump_as_bytes(body)
+        res = req.get_response(fakes.wsgi_app())
+        self.assertEqual(202, res.status_int)
+
     def test_attach_to_host(self):
         # using 'read-write' mode attach volume by default
         body = {'os-attach': {'host_name': 'fake_host',
                               'mountpoint': '/dev/vdc'}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
@@ -220,7 +238,7 @@ class VolumeActionsTest(test.TestCase):
         body = {'os-detach': {'attachment_id': 'fakeuuid'}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
@@ -263,18 +281,7 @@ class VolumeActionsTest(test.TestCase):
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
         req.headers["content-type"] = "application/json"
-        req.body = jsonutils.dumps(body)
-        res = req.get_response(fakes.wsgi_app())
-        self.assertEqual(400, res.status_int)
-
-        # Invalid request to attach volume to an instance and a host
-        body = {'os-attach': {'instance_uuid': 'fake',
-                              'host_name': 'fake_host',
-                              'mountpoint': '/dev/vdc'}}
-        req = webob.Request.blank('/v2/fake/volumes/1/action')
-        req.method = "POST"
-        req.headers["content-type"] = "application/json"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -285,7 +292,7 @@ class VolumeActionsTest(test.TestCase):
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
         req.headers["content-type"] = "application/json"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
         body = {'os-attach': {'host_name': 'fake_host',
@@ -294,7 +301,7 @@ class VolumeActionsTest(test.TestCase):
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
         req.headers["content-type"] = "application/json"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -307,7 +314,7 @@ class VolumeActionsTest(test.TestCase):
         body = {'os-begin_detaching': {'fake': 'fake'}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
@@ -322,7 +329,7 @@ class VolumeActionsTest(test.TestCase):
         body = {'os-roll_detaching': {'fake': 'fake'}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
@@ -337,12 +344,28 @@ class VolumeActionsTest(test.TestCase):
         body = {'os-extend': {'new_size': 5}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(202, res.status_int)
 
+    def test_extend_volume_invalid_status(self):
+        def fake_extend_volume(*args, **kwargs):
+            msg = "Volume status must be available"
+            raise exception.InvalidVolume(reason=msg)
+        self.stubs.Set(volume.api.API, 'extend',
+                       fake_extend_volume)
+
+        body = {'os-extend': {'new_size': 5}}
+        req = webob.Request.blank('/v2/fake/volumes/1/action')
+        req.method = "POST"
+        req.body = jsonutils.dump_as_bytes(body)
+        req.headers["content-type"] = "application/json"
+
+        res = req.get_response(fakes.wsgi_app())
+        self.assertEqual(400, res.status_int)
+
     def test_update_readonly_flag(self):
         def fake_update_readonly_flag(*args, **kwargs):
             return {}
@@ -355,7 +378,7 @@ class VolumeActionsTest(test.TestCase):
                 body = {"os-update_readonly_flag": {}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
             res = req.get_response(fakes.wsgi_app())
             self.assertEqual(return_code, res.status_int)
@@ -378,7 +401,7 @@ class VolumeActionsTest(test.TestCase):
                 body = {"os-set_bootable": {}}
             req = webob.Request.blank('/v2/fake/volumes/1/action')
             req.method = "POST"
-            req.body = jsonutils.dumps(body)
+            req.body = jsonutils.dump_as_bytes(body)
             req.headers["content-type"] = "application/json"
             res = req.get_response(fakes.wsgi_app())
             self.assertEqual(return_code, res.status_int)
@@ -426,7 +449,7 @@ class VolumeRetypeActionsTest(VolumeActionsTest):
         req.method = 'POST'
         req.headers['content-type'] = 'application/json'
         retype_body = {'new_type': new_type, 'migration_policy': 'never'}
-        req.body = jsonutils.dumps({'os-retype': retype_body})
+        req.body = jsonutils.dump_as_bytes({'os-retype': retype_body})
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(expected_status, res.status_int)
 
@@ -445,7 +468,7 @@ class VolumeRetypeActionsTest(VolumeActionsTest):
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
         req.headers['content-type'] = 'application/json'
-        req.body = jsonutils.dumps({'os-retype': None})
+        req.body = jsonutils.dump_as_bytes({'os-retype': None})
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -455,7 +478,7 @@ class VolumeRetypeActionsTest(VolumeActionsTest):
         req.method = 'POST'
         req.headers['content-type'] = 'application/json'
         retype_body = {'new_type': 'foo', 'migration_policy': 'invalid'}
-        req.body = jsonutils.dumps({'os-retype': retype_body})
+        req.body = jsonutils.dump_as_bytes({'os-retype': retype_body})
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -541,6 +564,7 @@ class VolumeImageActionsTest(test.TestCase):
         self.controller = volume_actions.VolumeActionsController()
 
         self.stubs.Set(volume_api.API, 'get', stub_volume_get)
+        self.context = context.RequestContext('fake', 'fake', is_admin=False)
 
     def _get_os_volume_upload_image(self):
         vol = {
@@ -595,12 +619,13 @@ class VolumeImageActionsTest(test.TestCase):
                      'status': 'uploading',
                      'display_description': 'displaydesc',
                      'size': 1,
-                     'volume_type': {'name': 'vol_type_name'},
+                     'volume_type': fake_volume.fake_db_volume_type(
+                         name='vol_type_name'),
                      'image_id': 1,
                      'container_format': 'bare',
                      'disk_format': 'raw',
                      'image_name': 'image_name'}}
-        self.assertDictMatch(res_dict, expected)
+        self.assertDictMatch(expected, res_dict)
 
     def test_copy_volume_to_image_volumenotfound(self):
         def stub_volume_get_raise_exc(self, context, volume_id):
@@ -690,7 +715,7 @@ class VolumeImageActionsTest(test.TestCase):
         req = webob.Request.blank('/v2/tenant1/volumes/%s/action' % id)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
-        req.body = json.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -704,7 +729,7 @@ class VolumeImageActionsTest(test.TestCase):
         req = webob.Request.blank('/v2/tenant1/volumes/%s/action' % id)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
-        req.body = json.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(400, res.status_int)
 
@@ -773,12 +798,14 @@ class VolumeImageActionsTest(test.TestCase):
                         expected_res = {
                             'os-volume_upload_image': {
                                 'id': id,
-                                'updated_at': datetime.datetime(1900, 1, 1,
-                                                                1, 1, 1),
+                                'updated_at': datetime.datetime(
+                                    1900, 1, 1, 1, 1, 1,
+                                    tzinfo=iso8601.iso8601.Utc()),
                                 'status': 'uploading',
                                 'display_description': 'displaydesc',
                                 'size': 1,
-                                'volume_type': {'name': 'vol_type_name'},
+                                'volume_type': fake_volume.fake_db_volume_type(
+                                    name='vol_type_name'),
                                 'image_id': 1,
                                 'container_format': 'bare',
                                 'disk_format': 'raw',
@@ -786,7 +813,7 @@ class VolumeImageActionsTest(test.TestCase):
                             }
                         }
 
-                        self.assertDictMatch(res_dict, expected_res)
+                        self.assertDictMatch(expected_res, res_dict)
 
     def test_copy_volume_to_image_without_glance_metadata(self):
         """Test create image from volume if volume is created without image.
@@ -829,12 +856,14 @@ class VolumeImageActionsTest(test.TestCase):
                         expected_res = {
                             'os-volume_upload_image': {
                                 'id': id,
-                                'updated_at': datetime.datetime(1900, 1, 1,
-                                                                1, 1, 1),
+                                'updated_at': datetime.datetime(
+                                    1900, 1, 1, 1, 1, 1,
+                                    tzinfo=iso8601.iso8601.Utc()),
                                 'status': 'uploading',
                                 'display_description': 'displaydesc',
                                 'size': 1,
-                                'volume_type': {'name': 'vol_type_name'},
+                                'volume_type': fake_volume.fake_db_volume_type(
+                                    name='vol_type_name'),
                                 'image_id': 1,
                                 'container_format': 'bare',
                                 'disk_format': 'raw',
@@ -842,7 +871,7 @@ class VolumeImageActionsTest(test.TestCase):
                             }
                         }
 
-                        self.assertDictMatch(res_dict, expected_res)
+                        self.assertDictMatch(expected_res, res_dict)
 
     def test_copy_volume_to_image_without_protected_prop(self):
         """Test protected property is not defined with the root image."""
@@ -882,12 +911,14 @@ class VolumeImageActionsTest(test.TestCase):
                         expected_res = {
                             'os-volume_upload_image': {
                                 'id': id,
-                                'updated_at': datetime.datetime(1900, 1, 1,
-                                                                1, 1, 1),
+                                'updated_at': datetime.datetime(
+                                    1900, 1, 1, 1, 1, 1,
+                                    tzinfo=iso8601.iso8601.Utc()),
                                 'status': 'uploading',
                                 'display_description': 'displaydesc',
                                 'size': 1,
-                                'volume_type': {'name': 'vol_type_name'},
+                                'volume_type': fake_volume.fake_db_volume_type(
+                                    name='vol_type_name'),
                                 'image_id': 1,
                                 'container_format': 'bare',
                                 'disk_format': 'raw',
@@ -895,7 +926,7 @@ class VolumeImageActionsTest(test.TestCase):
                             }
                         }
 
-                        self.assertDictMatch(res_dict, expected_res)
+                        self.assertDictMatch(expected_res, res_dict)
 
     def test_copy_volume_to_image_without_core_prop(self):
         """Test glance_core_properties defined in cinder.conf is empty."""
@@ -928,12 +959,14 @@ class VolumeImageActionsTest(test.TestCase):
                     expected_res = {
                         'os-volume_upload_image': {
                             'id': id,
-                            'updated_at': datetime.datetime(1900, 1, 1,
-                                                            1, 1, 1),
+                            'updated_at': datetime.datetime(
+                                1900, 1, 1, 1, 1, 1,
+                                tzinfo=iso8601.iso8601.Utc()),
                             'status': 'uploading',
                             'display_description': 'displaydesc',
                             'size': 1,
-                            'volume_type': {'name': 'vol_type_name'},
+                            'volume_type': fake_volume.fake_db_volume_type(
+                                name='vol_type_name'),
                             'image_id': 1,
                             'container_format': 'bare',
                             'disk_format': 'raw',
@@ -941,4 +974,48 @@ class VolumeImageActionsTest(test.TestCase):
                         }
                     }
 
-                    self.assertDictMatch(res_dict, expected_res)
+                    self.assertDictMatch(expected_res, res_dict)
+
+    @mock.patch.object(volume_api.API, "get_volume_image_metadata")
+    @mock.patch.object(glance.GlanceImageService, "create")
+    @mock.patch.object(volume_api.API, "get")
+    @mock.patch.object(volume_api.API, "update")
+    @mock.patch.object(volume_rpcapi.VolumeAPI, "copy_volume_to_image")
+    def test_copy_volume_to_image_volume_type_none(
+            self,
+            mock_copy_volume_to_image,
+            mock_update,
+            mock_get,
+            mock_create,
+            mock_get_volume_image_metadata):
+        """Test create image from volume with none type volume."""
+
+        db_volume = fake_volume.fake_db_volume()
+        volume_obj = objects.Volume._from_db_object(self.context,
+                                                    objects.Volume(),
+                                                    db_volume)
+
+        mock_create.side_effect = self.fake_image_service_create
+        mock_get.return_value = volume_obj
+        mock_copy_volume_to_image.side_effect = (
+            self.fake_rpc_copy_volume_to_image)
+
+        req = fakes.HTTPRequest.blank('/v2/tenant1/volumes/%s/action' % id)
+        body = self._get_os_volume_upload_image()
+        res_dict = self.controller._volume_upload_image(req, id, body)
+        expected_res = {
+            'os-volume_upload_image': {
+                'id': fake.volume_id,
+                'updated_at': None,
+                'status': 'uploading',
+                'display_description': None,
+                'size': 1,
+                'volume_type': None,
+                'image_id': 1,
+                'container_format': u'bare',
+                'disk_format': u'raw',
+                'image_name': u'image_name'
+            }
+        }
+
+        self.assertDictMatch(expected_res, res_dict)
