@@ -12,7 +12,6 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-import json
 import uuid
 from xml.dom import minidom
 
@@ -26,12 +25,14 @@ from cinder.api.openstack import wsgi
 from cinder import context
 from cinder import db
 from cinder import exception
+from cinder import objects
 from cinder import test
 from cinder.tests.unit.api import fakes
+from cinder.tests.unit import fake_volume
 from cinder import volume
 
 
-def fake_volume_get(*args, **kwargs):
+def fake_db_volume_get(*args, **kwargs):
     return {
         'id': 'fake',
         'host': 'host001',
@@ -39,21 +40,25 @@ def fake_volume_get(*args, **kwargs):
         'size': 5,
         'availability_zone': 'somewhere',
         'created_at': timeutils.utcnow(),
-        'attach_status': None,
         'display_name': 'anothervolume',
         'display_description': 'Just another volume!',
         'volume_type_id': None,
         'snapshot_id': None,
         'project_id': 'fake',
+        'migration_status': None,
+        '_name_id': 'fake2',
+        'attach_status': 'detached',
     }
 
 
+def fake_volume_api_get(*args, **kwargs):
+    ctx = context.RequestContext('admin', 'fake', True)
+    db_volume = fake_db_volume_get()
+    return fake_volume.fake_volume_obj(ctx, **db_volume)
+
+
 def fake_volume_get_all(*args, **kwargs):
-    return [fake_volume_get()]
-
-
-def fake_volume_get_all_empty(*args, **kwargs):
-    return []
+    return objects.VolumeList(objects=[fake_volume_api_get()])
 
 
 fake_image_metadata = {
@@ -94,13 +99,12 @@ class VolumeImageMetadataTest(test.TestCase):
 
     def setUp(self):
         super(VolumeImageMetadataTest, self).setUp()
-        self.stubs.Set(volume.api.API, 'get', fake_volume_get)
+        self.stubs.Set(volume.api.API, 'get', fake_volume_api_get)
         self.stubs.Set(volume.api.API, 'get_all', fake_volume_get_all)
         self.stubs.Set(volume.api.API, 'get_volume_image_metadata',
                        fake_get_volume_image_metadata)
         self.stubs.Set(volume.api.API, 'get_volumes_image_metadata',
                        fake_get_volumes_image_metadata)
-        self.stubs.Set(db, 'volume_get', fake_volume_get)
         self.UUID = uuid.uuid4()
         self.controller = (volume_image_metadata.
                            VolumeImageMetadataController())
@@ -112,12 +116,12 @@ class VolumeImageMetadataTest(test.TestCase):
         return res
 
     def _get_image_metadata(self, body):
-        return json.loads(body)['volume']['volume_image_metadata']
+        return jsonutils.loads(body)['volume']['volume_image_metadata']
 
     def _get_image_metadata_list(self, body):
         return [
             volume['volume_image_metadata']
-            for volume in json.loads(body)['volumes']
+            for volume in jsonutils.loads(body)['volumes']
         ]
 
     def _create_volume_and_glance_metadata(self):
@@ -146,18 +150,6 @@ class VolumeImageMetadataTest(test.TestCase):
         self.assertEqual(fake_image_metadata,
                          self._get_image_metadata_list(res.body)[0])
 
-    def test_list_detail_empty_volumes(self):
-        def fake_dont_call_this(*args, **kwargs):
-            fake_dont_call_this.called = True
-        fake_dont_call_this.called = False
-        self.stubs.Set(volume.api.API, 'get_list_volumes_image_metadata',
-                       fake_dont_call_this)
-        self.stubs.Set(volume.api.API, 'get_all', fake_volume_get_all_empty)
-
-        res = self._make_request('/v2/fake/volumes/detail')
-        self.assertEqual(200, res.status_int)
-        self.assertFalse(fake_dont_call_this.called)
-
     def test_list_detail_volumes_with_limit(self):
         ctxt = context.get_admin_context()
         db.volume_create(ctxt, {'id': 'fake', 'status': 'available',
@@ -179,13 +171,13 @@ class VolumeImageMetadataTest(test.TestCase):
         body = {"os-set_image_metadata": {"metadata": fake_image_metadata}}
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = "POST"
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(200, res.status_int)
         self.assertEqual(fake_image_metadata,
-                         json.loads(res.body)["metadata"])
+                         jsonutils.loads(res.body)["metadata"])
 
     def test_create_with_keys_case_insensitive(self):
         # If the keys in uppercase_and_lowercase, should return the one
@@ -208,13 +200,13 @@ class VolumeImageMetadataTest(test.TestCase):
 
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
         self.assertEqual(200, res.status_int)
         self.assertEqual(fake_image_metadata,
-                         json.loads(res.body)["metadata"])
+                         jsonutils.loads(res.body)["metadata"])
 
     def test_create_empty_body(self):
         req = fakes.HTTPRequest.blank('/v2/fake/volumes/1/action')
@@ -233,7 +225,7 @@ class VolumeImageMetadataTest(test.TestCase):
         body = {"os-set_image_metadata": {
             "metadata": {"image_name": "fake"}}
         }
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller.create, req, 1, body)
 
@@ -249,7 +241,7 @@ class VolumeImageMetadataTest(test.TestCase):
         }
 
         # Test for long key
-        req.body = jsonutils.dumps(data)
+        req.body = jsonutils.dump_as_bytes(data)
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.controller.create, req, 1, data)
 
@@ -257,7 +249,7 @@ class VolumeImageMetadataTest(test.TestCase):
         data = {"os-set_image_metadata": {
             "metadata": {"key": "v" * 260}}
         }
-        req.body = jsonutils.dumps(data)
+        req.body = jsonutils.dump_as_bytes(data)
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.controller.create, req, 1, data)
 
@@ -265,7 +257,7 @@ class VolumeImageMetadataTest(test.TestCase):
         data = {"os-set_image_metadata": {
             "metadata": {"": "value1"}}
         }
-        req.body = jsonutils.dumps(data)
+        req.body = jsonutils.dump_as_bytes(data)
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create, req, 1, data)
 
@@ -278,7 +270,7 @@ class VolumeImageMetadataTest(test.TestCase):
         }
         req = webob.Request.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         res = req.get_response(fakes.wsgi_app())
@@ -290,7 +282,7 @@ class VolumeImageMetadataTest(test.TestCase):
         }
         req = fakes.HTTPRequest.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
-        req.body = jsonutils.dumps(data)
+        req.body = jsonutils.dump_as_bytes(data)
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPNotFound,
@@ -305,12 +297,24 @@ class VolumeImageMetadataTest(test.TestCase):
         }
         req = fakes.HTTPRequest.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
-        req.body = jsonutils.dumps(body)
+        req.body = jsonutils.dump_as_bytes(body)
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller.delete, req, 1, body)
 
+    def test_show_image_metadata(self):
+        body = {"os-show_image_metadata": None}
+        req = webob.Request.blank('/v2/fake/volumes/1/action')
+        req.method = 'POST'
+        req.body = jsonutils.dump_as_bytes(body)
+        req.headers["content-type"] = "application/json"
+
+        res = req.get_response(fakes.wsgi_app())
+        self.assertEqual(200, res.status_int)
+        self.assertEqual(fake_image_metadata,
+                         jsonutils.loads(res.body)["metadata"])
+
 
 class ImageMetadataXMLDeserializer(common.MetadataXMLDeserializer):
     metadata_node_name = "volume_image_metadata"
@@ -337,5 +341,7 @@ class VolumeImageMetadataXMLTest(VolumeImageMetadataTest):
                 volume, 'volume_image_metadata'
             )
             for volume in volume_list]
-        return map(wsgi.MetadataXMLDeserializer().extract_metadata,
-                   image_metadata_list)
+
+        metadata_deserializer = wsgi.MetadataXMLDeserializer()
+        return [metadata_deserializer.extract_metadata(image_metadata)
+                for image_metadata in image_metadata_list]
