@@ -44,10 +44,10 @@ class LimiterTest(test.TestCase):
     def setUp(self):
         """Run before each test."""
         super(LimiterTest, self).setUp()
-        self.tiny = range(1)
-        self.small = range(10)
-        self.medium = range(1000)
-        self.large = range(10000)
+        self.tiny = list(range(1))
+        self.small = list(range(10))
+        self.medium = list(range(1000))
+        self.large = list(range(10000))
 
     def test_limiter_offset_zero(self):
         """Test offset key works with 0."""
@@ -80,6 +80,12 @@ class LimiterTest(test.TestCase):
         self.assertRaises(
             webob.exc.HTTPBadRequest, common.limited, self.tiny, req)
 
+    def test_limiter_offset_out_of_range(self):
+        """Test offset key works with a offset out of range."""
+        req = webob.Request.blank('/?offset=123456789012346456')
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, common.limited, self.tiny, req)
+
     def test_limiter_offset_bad(self):
         """Test offset key works with a BAD offset."""
         req = webob.Request.blank(u'/?offset=\u0020aa')
@@ -126,7 +132,7 @@ class LimiterTest(test.TestCase):
 
     def test_limiter_limit_and_offset(self):
         """Test request with both limit and offset."""
-        items = range(2000)
+        items = list(range(2000))
         req = webob.Request.blank('/?offset=1&limit=3')
         self.assertEqual(items[1:4], common.limited(items, req))
         req = webob.Request.blank('/?offset=3&limit=0')
@@ -135,10 +141,13 @@ class LimiterTest(test.TestCase):
         self.assertEqual(items[3:1003], common.limited(items, req))
         req = webob.Request.blank('/?offset=3000&limit=10')
         self.assertEqual([], common.limited(items, req))
+        req = webob.Request.blank('/?offset=30034522235674530&limit=10')
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, common.limited, items, req)
 
     def test_limiter_custom_max_limit(self):
         """Test a max_limit other than 1000."""
-        items = range(2000)
+        items = list(range(2000))
         req = webob.Request.blank('/?offset=1&limit=3')
         self.assertEqual(
             items[1:4], common.limited(items, req, max_limit=2000))
