@@ -569,7 +569,7 @@ class WsgiLimiterTest(BaseLimitTestSuite):
 
     def _request_data(self, verb, path):
         """Get data describing a limit request verb/path."""
-        return jsonutils.dumps({"verb": verb, "path": path})
+        return jsonutils.dump_as_bytes({"verb": verb, "path": path})
 
     def _request(self, verb, url, username=None):
         """Assert that POSTing to given url triggers given action.
@@ -636,9 +636,11 @@ class FakeHttplibSocket(object):
 
     def __init__(self, response_string):
         """Initialize new `FakeHttplibSocket`."""
-        self._buffer = six.StringIO(response_string)
+        if isinstance(response_string, six.text_type):
+            response_string = response_string.encode('utf-8')
+        self._buffer = six.BytesIO(response_string)
 
-    def makefile(self, _mode, _other):
+    def makefile(self, mode, *args):
         """Returns the socket's internal buffer."""
         return self._buffer
 
@@ -754,8 +756,9 @@ class WsgiLimiterProxyTest(BaseLimitTestSuite):
         delay, error = self.proxy.check_for_delay("GET", "/delayed")
         error = error.strip()
 
-        expected = ("60.00", "403 Forbidden\n\nOnly 1 GET request(s) can be "
-                    "made to /delayed every minute.")
+        expected = ("60.00",
+                    b"403 Forbidden\n\nOnly 1 GET request(s) can be "
+                    b"made to /delayed every minute.")
 
         self.assertEqual(expected, (delay, error))
 
@@ -806,7 +809,7 @@ class LimitsViewBuilderTest(test.TestCase):
 
         output = self.view_builder.build(self.rate_limits,
                                          self.absolute_limits)
-        self.assertDictMatch(output, expected_limits)
+        self.assertDictMatch(expected_limits, output)
 
     def test_build_limits_empty_limits(self):
         expected_limits = {"limits": {"rate": [],
@@ -815,7 +818,7 @@ class LimitsViewBuilderTest(test.TestCase):
         abs_limits = {}
         rate_limits = []
         output = self.view_builder.build(rate_limits, abs_limits)
-        self.assertDictMatch(output, expected_limits)
+        self.assertDictMatch(expected_limits, output)
 
 
 class LimitsXMLSerializationTest(test.TestCase):
@@ -827,7 +830,7 @@ class LimitsXMLSerializationTest(test.TestCase):
                    "absolute": {}}}
 
         output = serializer.serialize(fixture)
-        has_dec = output.startswith("<?xml version='1.0' encoding='UTF-8'?>")
+        has_dec = output.startswith(b"<?xml version='1.0' encoding='UTF-8'?>")
         self.assertTrue(has_dec)
 
     def test_index(self):
