@@ -149,10 +149,10 @@ class TestGlanceImageService(test.TestCase):
             'properties': {'instance_id': '42', 'user_id': 'fake'},
             'owner': None,
         }
-        self.assertDictMatch(image_meta, expected)
+        self.assertDictMatch(expected, image_meta)
 
         image_metas = self.service.detail(self.context)
-        self.assertDictMatch(image_metas[0], expected)
+        self.assertDictMatch(expected, image_metas[0])
 
     def test_create_without_instance_id(self):
         """Test Creating images without instance_id.
@@ -183,7 +183,7 @@ class TestGlanceImageService(test.TestCase):
             'owner': None,
         }
         actual = self.service.show(self.context, image_id)
-        self.assertDictMatch(actual, expected)
+        self.assertDictMatch(expected, actual)
 
     def test_create(self):
         fixture = self._make_fixture(name='test image')
@@ -254,7 +254,7 @@ class TestGlanceImageService(test.TestCase):
                 'owner': None,
             }
 
-            self.assertDictMatch(meta, expected)
+            self.assertDictMatch(expected, meta)
             i = i + 1
 
     def test_detail_limit(self):
@@ -310,7 +310,7 @@ class TestGlanceImageService(test.TestCase):
                 'deleted': None,
                 'owner': None,
             }
-            self.assertDictMatch(meta, expected)
+            self.assertDictMatch(expected, meta)
             i = i + 1
 
     def test_detail_invalid_marker(self):
@@ -554,7 +554,7 @@ class TestGlanceImageService(test.TestCase):
         self.flags(allowed_direct_url_schemes=['file'])
         self.flags(glance_api_version=2)
         self.service.download(self.context, image_id, writer)
-        self.assertEqual(None, mock_copyfileobj.call_args)
+        self.assertIsNone(mock_copyfileobj.call_args)
 
     def test_glance_client_image_id(self):
         fixture = self._make_fixture(name='test image')
