@@ -17,9 +17,12 @@ import mock
 from oslo_config import cfg
 
 from cinder import context
-from cinder.db.sqlalchemy import api
+from cinder import db
 import cinder.exception
+from cinder.objects import snapshot as obj_snap
+from cinder.objects import volume as obj_volume
 import cinder.test
+from cinder.tests.unit import fake_constants as fake
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import block_device
 from cinder.volume import utils as volutils
@@ -37,15 +40,14 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
         self.configuration.volume_dd_blocksize = 1234
         self.drv = block_device.BlockDeviceDriver(
             configuration=self.configuration,
-            host='localhost')
+            host='localhost', db=db)
 
     def test_initialize_connection(self):
-        TEST_VOLUME1 = {'host': 'localhost1',
-                        'provider_location': '1 2 3 /dev/loop1',
-                        'provider_auth': 'a b c',
-                        'attached_mode': 'rw',
-                        'id': 'fake-uuid'}
-
+        TEST_VOLUME1 = obj_volume.Volume(host='localhost1',
+                                         provider_location='1 2 3 /dev/loop1',
+                                         provider_auth='a b c',
+                                         attached_mode='rw',
+                                         id=fake.volume_id)
         TEST_CONNECTOR = {'host': 'localhost1'}
 
         data = self.drv.initialize_connection(TEST_VOLUME1, TEST_CONNECTOR)
@@ -57,11 +59,11 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
     @mock.patch('cinder.volume.driver.ISCSIDriver.initialize_connection')
     def test_initialize_connection_different_hosts(self, _init_conn):
         TEST_CONNECTOR = {'host': 'localhost1'}
-        TEST_VOLUME2 = {'host': 'localhost2',
-                        'provider_location': '1 2 3 /dev/loop2',
-                        'provider_auth': 'd e f',
-                        'attached_mode': 'rw',
-                        'id': 'fake-uuid-2'}
+        TEST_VOLUME2 = obj_volume.Volume(host='localhost2',
+                                         provider_location='1 2 3 /dev/loop2',
+                                         provider_auth='d e f',
+                                         attached_mode='rw',
+                                         id=fake.volume2_id)
         _init_conn.return_value = 'data'
 
         data = self.drv.initialize_connection(TEST_VOLUME2, TEST_CONNECTOR)
@@ -73,7 +75,7 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
                                   'target_iqn': '2',
                                   'target_lun': 3,
                                   'target_portal': '1',
-                                  'volume_id': 'fake-uuid-2'}}
+                                  'volume_id': fake.volume2_id}}
 
         self.assertEqual(expected_data['data'], data['data'])
 
@@ -82,45 +84,63 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
     @mock.patch('cinder.volume.utils.clear_volume')
     def test_delete_not_volume_provider_location(self, _clear_volume,
                                                  _local_path):
-        TEST_VOLUME2 = {'provider_location': None}
+        TEST_VOLUME2 = obj_volume.Volume(provider_location=None)
         self.drv.delete_volume(TEST_VOLUME2)
         _local_path.assert_called_once_with(TEST_VOLUME2)
 
     @mock.patch('os.path.exists', return_value=True)
     @mock.patch('cinder.volume.utils.clear_volume')
     def test_delete_volume_path_exist(self, _clear_volume, _exists):
-        TEST_VOLUME1 = {'provider_location': '1 2 3 /dev/loop1'}
+        TEST_VOLUME = obj_volume.Volume(name_id=fake.volume_name_id,
+                                        size=1,
+                                        provider_location='/dev/loop1',
+                                        display_name='vol1',
+                                        status='available')
 
         with mock.patch.object(self.drv, 'local_path',
                                return_value='/dev/loop1') as lp_mocked:
-            with mock.patch.object(self.drv, '_get_device_size',
-                                   return_value=1024) as gds_mocked:
+            with mock.patch.object(self.drv, '_get_devices_sizes',
+                                   return_value={'/dev/loop1': 1}) as \
+                    gds_mocked:
                 volutils.clear_volume(gds_mocked, lp_mocked)
 
-                self.drv.delete_volume(TEST_VOLUME1)
+                self.drv.delete_volume(TEST_VOLUME)
 
-                lp_mocked.assert_called_once_with(TEST_VOLUME1)
-                gds_mocked.assert_called_once_with('/dev/loop1')
+                lp_mocked.assert_called_once_with(TEST_VOLUME)
+                gds_mocked.assert_called_once_with(['/dev/loop1'])
 
         self.assertTrue(_exists.called)
         self.assertTrue(_clear_volume.called)
 
     def test_delete_path_is_not_in_list_of_available_devices(self):
-        TEST_VOLUME2 = {'provider_location': '1 2 3 /dev/loop0'}
+        TEST_VOLUME2 = obj_volume.Volume(provider_location='/dev/loop0')
         with mock.patch.object(self.drv, 'local_path',
                                return_value='/dev/loop0') as lp_mocked:
             self.drv.delete_volume(TEST_VOLUME2)
             lp_mocked.assert_called_once_with(TEST_VOLUME2)
 
+    def test__update_provider_location(self):
+        TEST_VOLUME = obj_volume.Volume(name_id=fake.volume_name_id,
+                                        size=1,
+                                        display_name='vol1')
+        with mock.patch.object(obj_volume.Volume, 'update') as update_mocked, \
+                mock.patch.object(obj_volume.Volume, 'save') as save_mocked:
+            self.drv._update_provider_location(TEST_VOLUME, 'dev_path')
+            self.assertEqual(1, update_mocked.call_count)
+            save_mocked.assert_called_once_with()
+
     def test_create_volume(self):
-        TEST_VOLUME = {'size': 1,
-                       'name': 'vol1'}
+        TEST_VOLUME = obj_volume.Volume(name_id=fake.volume_name_id,
+                                        size=1,
+                                        display_name='vol1')
 
         with mock.patch.object(self.drv, 'find_appropriate_size_device',
                                return_value='dev_path') as fasd_mocked:
-            result = self.drv.create_volume(TEST_VOLUME)
-            self.assertEqual({'provider_location': 'dev_path'}, result)
-            fasd_mocked.assert_called_once_with(TEST_VOLUME['size'])
+            with mock.patch.object(self.drv, '_update_provider_location') as \
+                    upl_mocked:
+                self.drv.create_volume(TEST_VOLUME)
+                fasd_mocked.assert_called_once_with(TEST_VOLUME.size)
+                upl_mocked.assert_called_once_with(TEST_VOLUME, 'dev_path')
 
     def test_update_volume_stats(self):
 
@@ -147,32 +167,40 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
 
     @mock.patch('cinder.volume.utils.copy_volume')
     def test_create_cloned_volume(self, _copy_volume):
-        TEST_SRC = {'id': '1',
-                    'size': 1,
-                    'provider_location': '1 2 3 /dev/loop1'}
-        TEST_VOLUME = {}
+        TEST_SRC = obj_volume.Volume(id=fake.volume_id,
+                                     name_id=fake.volume_name_id,
+                                     size=1,
+                                     provider_location='/dev/loop1')
+        TEST_VOLUME = obj_volume.Volume(name_id=fake.volume2_name_id,
+                                        size=1,
+                                        display_name='vol1')
 
         with mock.patch.object(self.drv, 'find_appropriate_size_device',
                                return_value='/dev/loop2') as fasd_mocked:
-            with mock.patch.object(self.drv, '_get_device_size',
-                                   return_value=1) as gds_mocked:
+            with mock.patch.object(self.drv, '_get_devices_sizes',
+                                   return_value={'/dev/loop2': 2}) as \
+                    gds_mocked:
                 with mock.patch.object(self.drv, 'local_path',
                                        return_value='/dev/loop1') as \
                         lp_mocked:
-                    volutils.copy_volume('/dev/loop1', fasd_mocked, 2048,
-                                         mock.sentinel,
-                                         execute=self.drv._execute)
-
-                    self.assertEqual({'provider_location': '/dev/loop2'},
-                                     self.drv.create_cloned_volume(TEST_VOLUME,
-                                                                   TEST_SRC))
-                    fasd_mocked.assert_called_once_with(TEST_SRC['size'])
-                    lp_mocked.assert_called_once_with(TEST_SRC)
-                    gds_mocked.assert_called_once_with('/dev/loop2')
+                    with mock.patch.object(self.drv,
+                                           '_update_provider_location') as \
+                            upl_mocked:
+                        volutils.copy_volume('/dev/loop1', fasd_mocked, 2,
+                                             mock.sentinel,
+                                             execute=self.drv._execute)
+                        self.drv.create_cloned_volume(TEST_VOLUME, TEST_SRC)
+                        fasd_mocked.assert_called_once_with(TEST_SRC.size)
+                        lp_mocked.assert_called_once_with(TEST_SRC)
+                        gds_mocked.assert_called_once_with(['/dev/loop2'])
+                        upl_mocked.assert_called_once_with(
+                            TEST_VOLUME, '/dev/loop2')
 
     @mock.patch.object(cinder.image.image_utils, 'fetch_to_raw')
     def test_copy_image_to_volume(self, _fetch_to_raw):
-        TEST_VOLUME = {'provider_location': '1 2 3 /dev/loop1', 'size': 1}
+        TEST_VOLUME = obj_volume.Volume(name_id=fake.volume_name_id,
+                                        size=1,
+                                        provider_location='/dev/loop1')
         TEST_IMAGE_SERVICE = "image_service"
         TEST_IMAGE_ID = "image_id"
 
@@ -187,7 +215,7 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
                                               1234, size=1)
 
     def test_copy_volume_to_image(self):
-        TEST_VOLUME = {'provider_location': '1 2 3 /dev/loop1'}
+        TEST_VOLUME = {'provider_location': '/dev/loop1'}
         TEST_IMAGE_SERVICE = "image_service"
         TEST_IMAGE_META = "image_meta"
 
@@ -207,15 +235,17 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
 
     def test_get_used_devices(self):
         TEST_VOLUME1 = {'host': 'localhost',
-                        'provider_location': '1 2 3 /dev/loop1'}
+                        'provider_location': '/dev/loop1'}
         TEST_VOLUME2 = {'host': 'localhost',
-                        'provider_location': '1 2 3 /dev/loop2'}
+                        'provider_location': '/dev/loop2'}
 
         def fake_local_path(vol):
             return vol['provider_location'].split()[-1]
 
-        with mock.patch.object(api, 'volume_get_all_by_host',
-                               return_value=[TEST_VOLUME1, TEST_VOLUME2]):
+        with mock.patch.object(obj_volume.VolumeList, 'get_all_by_host',
+                               return_value=[TEST_VOLUME1, TEST_VOLUME2]),\
+                mock.patch.object(obj_snap.SnapshotList, 'get_by_host',
+                                  return_value=[]):
             with mock.patch.object(context, 'get_admin_context'):
                 with mock.patch.object(self.drv, 'local_path',
                                        return_value=fake_local_path):
@@ -224,19 +254,24 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
                     self.assertEqual(set([path1, path2]),
                                      self.drv._get_used_devices())
 
-    def test_get_device_size(self):
-        dev_path = '/dev/loop1'
-        out = '2048'
+    def test_get_devices_sizes(self):
+        dev_paths = ['/dev/loop1', '/dev/loop2', '/dev/loop3']
+        out = '4294967296\n2147483648\n3221225472\nn'
         with mock.patch.object(self.drv,
                                '_execute',
                                return_value=(out, None)) as _execute:
-            self.assertEqual(1, self.drv._get_device_size(dev_path))
-            _execute.assert_called_once_with('blockdev', '--getsz', dev_path,
-                                             run_as_root=True)
+            actual = self.drv._get_devices_sizes(dev_paths)
+            self.assertEqual(3, len(actual))
+            self.assertEqual({'/dev/loop1': 4096, '/dev/loop2': 2048,
+                              '/dev/loop3': 3072}, actual)
+            _execute.assert_called_once_with('blockdev', '--getsize64',
+                                             *dev_paths, run_as_root=True)
 
     def test_devices_sizes(self):
-        with mock.patch.object(self.drv, '_get_device_size') as _get_dvc_size:
-            _get_dvc_size.return_value = 1
+        with mock.patch.object(self.drv, '_get_devices_sizes') as \
+                _get_dvc_size:
+            _get_dvc_size.return_value = {'/dev/loop1': 1, '/dev/loop2': 1}
+            self.assertEqual(2, len(self.drv._devices_sizes()))
             self.assertEqual({'/dev/loop1': 1, '/dev/loop2': 1},
                              self.drv._devices_sizes())
 
@@ -245,19 +280,19 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
         with mock.patch.object(self.drv, '_devices_sizes') as _dvc_sizes:
             with mock.patch.object(self.drv, '_get_used_devices') as \
                     _get_used_dvc:
-                _dvc_sizes.return_value = {'/dev/loop1': 1024,
-                                           '/dev/loop2': 1024}
+                _dvc_sizes.return_value = {'/dev/loop1': 1,
+                                           '/dev/loop2': 1}
                 _get_used_dvc.return_value = set(['/dev/loop1', '/dev/loop2'])
                 self.assertRaises(cinder.exception.CinderException,
                                   self.drv.find_appropriate_size_device, size)
 
     def test_find_appropriate_size_device_not_big_enough_disk(self):
-        size = 2
+        size = 2948
         with mock.patch.object(self.drv, '_devices_sizes') as _dvc_sizes:
             with mock.patch.object(self.drv, '_get_used_devices') as \
                     _get_used_dvc:
                 _dvc_sizes.return_value = {'/dev/loop1': 1024,
-                                           '/dev/loop2': 1024}
+                                           '/dev/loop2': 1924}
                 _get_used_dvc.return_value = set(['/dev/loop1'])
                 self.assertRaises(cinder.exception.CinderException,
                                   self.drv.find_appropriate_size_device, size)
@@ -272,3 +307,112 @@ class TestBlockDeviceDriver(cinder.test.TestCase):
                 _get_used_dvc.return_value = set()
                 self.assertEqual('/dev/loop2',
                                  self.drv.find_appropriate_size_device(size))
+
+    def test_extend_volume_exists(self):
+        TEST_VOLUME = {'name': 'vol1', 'id': 123}
+        with mock.patch.object(self.drv, '_get_devices_sizes',
+                               return_value={'/dev/loop1': 1024}) as \
+                mock_get_size:
+            with mock.patch.object(self.drv, 'local_path',
+                                   return_value='/dev/loop1') as lp_mocked:
+                self.assertRaises(cinder.exception.CinderException,
+                                  self.drv.extend_volume, TEST_VOLUME, 2)
+                lp_mocked.assert_called_once_with(TEST_VOLUME)
+                mock_get_size.assert_called_once_with(['/dev/loop1'])
+
+    @mock.patch('cinder.volume.utils.copy_volume')
+    def test_create_snapshot(self, _copy_volume):
+        TEST_VOLUME = obj_volume.Volume(id=fake.volume_id,
+                                        name_id=fake.volume_name_id,
+                                        size=1,
+                                        display_name='vol1',
+                                        status='available',
+                                        provider_location='/dev/loop1')
+        TEST_SNAP = obj_snap.Snapshot(id=fake.snapshot_id,
+                                      volume_id=fake.volume_id,
+                                      volume_size=1024,
+                                      provider_location='/dev/loop2',
+                                      volume=TEST_VOLUME)
+
+        with mock.patch.object(self.drv, 'find_appropriate_size_device',
+                               return_value='/dev/loop2') as fasd_mocked:
+            with mock.patch.object(self.drv, '_get_devices_sizes',
+                                   return_value={'/dev/loop2': 1024}) as \
+                    gds_mocked:
+                with mock.patch.object(self.drv,
+                                       '_update_provider_location') as \
+                        upl_mocked:
+                    volutils.copy_volume('/dev/loop1', fasd_mocked, 1024,
+                                         mock.sentinel,
+                                         execute=self.drv._execute)
+                    self.drv.create_snapshot(TEST_SNAP)
+                    fasd_mocked.assert_called_once_with(TEST_SNAP.volume_size)
+                    gds_mocked.assert_called_once_with(['/dev/loop2'])
+                    upl_mocked.assert_called_once_with(
+                        TEST_SNAP, '/dev/loop2')
+
+    def test_create_snapshot_with_not_available_volume(self):
+        TEST_VOLUME = obj_volume.Volume(id=fake.volume_id,
+                                        name_id=fake.volume_name_id,
+                                        size=1,
+                                        display_name='vol1',
+                                        status='in use',
+                                        provider_location='/dev/loop1')
+        TEST_SNAP = obj_snap.Snapshot(id=fake.snapshot_id,
+                                      volume_id=fake.volume_id,
+                                      volume_size=1024,
+                                      provider_location='/dev/loop2',
+                                      volume=TEST_VOLUME)
+
+        self.assertRaises(cinder.exception.CinderException,
+                          self.drv.create_snapshot, TEST_SNAP)
+
+    @mock.patch('cinder.volume.utils.copy_volume')
+    def test_create_volume_from_snapshot(self, _copy_volume):
+        TEST_SNAP = obj_snap.Snapshot(volume_id=fake.volume_id,
+                                      volume_size=1024,
+                                      provider_location='/dev/loop1')
+        TEST_VOLUME = obj_volume.Volume(id=fake.volume_id,
+                                        name_id=fake.volume_name_id,
+                                        size=1,
+                                        display_name='vol1',
+                                        provider_location='/dev/loop2')
+
+        with mock.patch.object(self.drv, 'find_appropriate_size_device',
+                               return_value='/dev/loop2') as fasd_mocked:
+            with mock.patch.object(self.drv, '_get_devices_sizes',
+                                   return_value={'/dev/loop2': 1024}) as \
+                    gds_mocked:
+                with mock.patch.object(self.drv,
+                                       '_update_provider_location') as \
+                        upl_mocked:
+                    volutils.copy_volume('/dev/loop1', fasd_mocked, 1024,
+                                         mock.sentinel,
+                                         execute=self.drv._execute)
+                    self.drv.create_volume_from_snapshot(
+                        TEST_VOLUME, TEST_SNAP)
+                    fasd_mocked.assert_called_once_with(
+                        TEST_SNAP.volume_size)
+                    gds_mocked.assert_called_once_with(['/dev/loop2'])
+                    upl_mocked.assert_called_once_with(
+                        TEST_VOLUME, '/dev/loop2')
+
+    @mock.patch('os.path.exists', return_value=True)
+    @mock.patch('cinder.volume.utils.clear_volume')
+    def test_delete_snapshot(self, _clear_volume, _exists):
+        TEST_SNAP = obj_snap.Snapshot(volume_id=fake.volume_id,
+                                      provider_location='/dev/loop1',
+                                      status='available')
+
+        with mock.patch.object(self.drv, 'local_path',
+                               return_value='/dev/loop1') as lp_mocked:
+            with mock.patch.object(self.drv, '_get_devices_sizes',
+                                   return_value={'/dev/loop1': 1}) as \
+                    gds_mocked:
+                volutils.clear_volume(gds_mocked, lp_mocked)
+                self.drv.delete_snapshot(TEST_SNAP)
+                lp_mocked.assert_called_once_with(TEST_SNAP)
+                gds_mocked.assert_called_once_with(['/dev/loop1'])
+
+        self.assertTrue(_exists.called)
+        self.assertTrue(_clear_volume.called)
