@@ -62,7 +62,7 @@ class HackingTestCase(test.TestCase):
                  'Line 6\n', 'Line 7\n', 'Line 8\n', 'Line 9\n', 'Line 10\n',
                  'Line 11\n']
 
-        self.assertEqual(None, checks.no_vi_headers(
+        self.assertIsNone(checks.no_vi_headers(
             "Test string foo", 1, lines))
         self.assertEqual(2, len(list(checks.no_vi_headers(
             "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
@@ -70,11 +70,11 @@ class HackingTestCase(test.TestCase):
         self.assertEqual(2, len(list(checks.no_vi_headers(
             "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
             8, lines))))
-        self.assertEqual(None, checks.no_vi_headers(
+        self.assertIsNone(checks.no_vi_headers(
             "Test end string for vi",
             9, lines))
         # vim header outside of boundary (first/last 5 lines)
-        self.assertEqual(None, checks.no_vi_headers(
+        self.assertIsNone(checks.no_vi_headers(
             "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
             6, lines))
 
@@ -105,7 +105,7 @@ class HackingTestCase(test.TestCase):
             "msg = _('My message')",
             "cinder.tests.unit/other_files.py"))))
         self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
-            "from cinder.i18n import _, _LW",
+            "from cinder.i18n import _LE, _, _LW",
             "cinder.tests.unit/other_files2.py"))))
         self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
@@ -120,6 +120,12 @@ class HackingTestCase(test.TestCase):
         self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "LOG.info('My info message')",
             "cinder.tests.unit/other_files4.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
+            "from cinder.i18n import _LW",
+            "cinder.tests.unit/other_files5.py"))))
+        self.assertEqual(1, len(list(checks.check_explicit_underscore_import(
+            "msg = _('My message')",
+            "cinder.tests.unit/other_files5.py"))))
 
     # We are patching pep8 so that only the check under test is actually
     # installed.
@@ -175,6 +181,40 @@ class HackingTestCase(test.TestCase):
         self._assert_has_errors(code, checker,
                                 expected_errors=[(4, 37, 'C310')])
 
+    def test_opt_type_registration_args(self):
+        checker = checks.CheckOptRegistrationArgs
+        code = """
+               CONF.register_opts([opt1, opt2, opt3])
+               CONF.register_opts((opt4, opt5))
+               CONF.register_opt(lonely_opt)
+               CONF.register_opts([OPT1, OPT2], group="group_of_opts")
+               CONF.register_opt(single_opt, group=blah)
+               """
+        self._assert_has_no_errors(code, checker)
+
+        code = """
+               CONF.register_opt([opt4, opt5, opt6])
+               CONF.register_opt((opt7, opt8))
+               CONF.register_opts(lonely_opt)
+               CONF.register_opt((an_opt, another_opt))
+               """
+        self._assert_has_errors(code, checker,
+                                expected_errors=[(1, 18, 'C311'),
+                                                 (2, 19, 'C311'),
+                                                 (3, 19, 'C311'),
+                                                 (4, 19, 'C311')])
+
+        code = """
+               CONF.register_opt(single_opt)
+               CONF.register_opts(other_opt)
+               CONF.register_opt(multiple_opts)
+               tuple_opts = (one_opt, two_opt)
+               CONF.register_opts(tuple_opts)
+               """
+        self._assert_has_errors(code, checker,
+                                expected_errors=[(2, 19, 'C311'),
+                                                 (3, 18, 'C311')])
+
     def test_str_unicode_exception(self):
 
         checker = checks.CheckForStrUnicodeExc
@@ -380,6 +420,20 @@ class HackingTestCase(test.TestCase):
         self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
             "      self._render_dict(xml, data_el, data.__dict__)"))))
 
+    def test_validate_assertIsNone(self):
+        test_value = None
+        self.assertEqual(0, len(list(checks.validate_assertIsNone(
+            "assertIsNone(None)"))))
+        self.assertEqual(1, len(list(checks.validate_assertIsNone(
+            "assertEqual(None, %s)" % test_value))))
+
+    def test_validate_assertTrue(self):
+        test_value = True
+        self.assertEqual(0, len(list(checks.validate_assertTrue(
+            "assertTrue(True)"))))
+        self.assertEqual(1, len(list(checks.validate_assertTrue(
+            "assertEqual(True, %s)" % test_value))))
+
     @ddt.unpack
     @ddt.data(
         (1, 'LOG.info', "cinder/tests/unit/fake.py", False),
