@@ -17,6 +17,7 @@
 Test suite for VMware VMDK driver volumeops module.
 """
 
+import ddt
 import mock
 from oslo_utils import units
 from oslo_vmware import exceptions
@@ -27,6 +28,7 @@ from cinder.volume.drivers.vmware import exceptions as vmdk_exceptions
 from cinder.volume.drivers.vmware import volumeops
 
 
+@ddt.ddt
 class VolumeOpsTestCase(test.TestCase):
     """Unit tests for volumeops module."""
 
@@ -945,7 +947,7 @@ class VolumeOpsTestCase(test.TestCase):
                 del obj.eagerlyScrub
             elif (type == "ns0:VirtualMachineRelocateSpec" and
                   delete_disk_attribute):
-                    del obj.disk
+                del obj.disk
             else:
                 pass
             return obj
@@ -1221,82 +1223,64 @@ class VolumeOpsTestCase(test.TestCase):
                                              disk_device)
         self._verify_extra_config(ret.config.extraConfig, key, value)
 
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                '_get_folder')
     @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
                 '_get_clone_spec')
-    def test_clone_backing(self, get_clone_spec):
-        folder = mock.Mock(name='folder', spec=object)
-        folder._type = 'Folder'
-        task = mock.sentinel.task
-        self.session.invoke_api.side_effect = [folder, task, folder, task,
-                                               folder, task]
-        task_info = mock.Mock(spec=object)
-        task_info.result = mock.sentinel.new_backing
-        self.session.wait_for_task.return_value = task_info
+    def _test_clone_backing(
+            self, clone_type, folder, get_clone_spec, get_folder):
+        backing_folder = mock.sentinel.backing_folder
+        get_folder.return_value = backing_folder
+
         clone_spec = mock.sentinel.clone_spec
         get_clone_spec.return_value = clone_spec
-        # Test non-linked clone_backing
+
+        task = mock.sentinel.task
+        self.session.invoke_api.return_value = task
+
+        clone = mock.sentinel.clone
+        self.session.wait_for_task.return_value = mock.Mock(result=clone)
+
         name = mock.sentinel.name
-        backing = mock.Mock(spec=object)
-        backing._type = 'VirtualMachine'
+        backing = mock.sentinel.backing
         snapshot = mock.sentinel.snapshot
-        clone_type = "anything-other-than-linked"
-        datastore = mock.sentinel.datstore
-        ret = self.vops.clone_backing(name, backing, snapshot, clone_type,
-                                      datastore)
-        # verify calls
-        self.assertEqual(mock.sentinel.new_backing, ret)
-        disk_move_type = 'moveAllDiskBackingsAndDisallowSharing'
-        get_clone_spec.assert_called_with(
-            datastore, disk_move_type, snapshot, backing, None, host=None,
-            resource_pool=None, extra_config=None)
-        expected = [mock.call(vim_util, 'get_object_property',
-                              self.session.vim, backing, 'parent'),
-                    mock.call(self.session.vim, 'CloneVM_Task', backing,
-                              folder=folder, name=name, spec=clone_spec)]
-        self.assertEqual(expected, self.session.invoke_api.mock_calls)
-
-        # Test linked clone_backing
-        clone_type = volumeops.LINKED_CLONE_TYPE
-        self.session.invoke_api.reset_mock()
-        ret = self.vops.clone_backing(name, backing, snapshot, clone_type,
-                                      datastore)
-        # verify calls
-        self.assertEqual(mock.sentinel.new_backing, ret)
-        disk_move_type = 'createNewChildDiskBacking'
-        get_clone_spec.assert_called_with(
-            datastore, disk_move_type, snapshot, backing, None, host=None,
-            resource_pool=None, extra_config=None)
-        expected = [mock.call(vim_util, 'get_object_property',
-                              self.session.vim, backing, 'parent'),
-                    mock.call(self.session.vim, 'CloneVM_Task', backing,
-                              folder=folder, name=name, spec=clone_spec)]
-        self.assertEqual(expected, self.session.invoke_api.mock_calls)
-
-        # Test with optional params (disk_type, host, resource_pool and
-        # extra_config).
-        clone_type = None
-        disk_type = 'thin'
+        datastore = mock.sentinel.datastore
+        disk_type = mock.sentinel.disk_type
         host = mock.sentinel.host
-        rp = mock.sentinel.rp
+        resource_pool = mock.sentinel.resource_pool
         extra_config = mock.sentinel.extra_config
-        self.session.invoke_api.reset_mock()
-        ret = self.vops.clone_backing(name, backing, snapshot, clone_type,
-                                      datastore, disk_type, host, rp,
-                                      extra_config)
+        ret = self.vops.clone_backing(
+            name, backing, snapshot, clone_type, datastore,
+            disk_type=disk_type, host=host, resource_pool=resource_pool,
+            extra_config=extra_config, folder=folder)
 
-        self.assertEqual(mock.sentinel.new_backing, ret)
-        disk_move_type = 'moveAllDiskBackingsAndDisallowSharing'
-        get_clone_spec.assert_called_with(
-            datastore, disk_move_type, snapshot, backing, disk_type, host=host,
-            resource_pool=rp, extra_config=extra_config)
-        expected = [mock.call(vim_util, 'get_object_property',
-                              self.session.vim, backing, 'parent'),
-                    mock.call(self.session.vim, 'CloneVM_Task', backing,
-                              folder=folder, name=name, spec=clone_spec)]
-        self.assertEqual(expected, self.session.invoke_api.mock_calls)
+        if folder:
+            self.assertFalse(get_folder.called)
+        else:
+            get_folder.assert_called_once_with(backing)
 
-        # Clear side effects.
-        self.session.invoke_api.side_effect = None
+        if clone_type == 'linked':
+            exp_disk_move_type = 'createNewChildDiskBacking'
+        else:
+            exp_disk_move_type = 'moveAllDiskBackingsAndDisallowSharing'
+        get_clone_spec.assert_called_once_with(
+            datastore, exp_disk_move_type, snapshot, backing, disk_type,
+            host=host, resource_pool=resource_pool, extra_config=extra_config)
+
+        exp_folder = folder if folder else backing_folder
+        self.session.invoke_api.assert_called_once_with(
+            self.session.vim, 'CloneVM_Task', backing, folder=exp_folder,
+            name=name, spec=clone_spec)
+
+        self.session.wait_for_task.assert_called_once_with(task)
+        self.assertEqual(clone, ret)
+
+    @ddt.data('linked', 'full')
+    def test_clone_backing(self, clone_type):
+        self._test_clone_backing(clone_type, mock.sentinel.folder)
+
+    def test_clone_backing_with_empty_folder(self):
+        self._test_clone_backing('linked', None)
 
     @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
                 '_create_specs_for_disk_add')
@@ -1326,6 +1310,39 @@ class VolumeOpsTestCase(test.TestCase):
                                                         spec=reconfig_spec)
         self.session.wait_for_task.assert_called_once_with(task)
 
+    def test_create_spec_for_disk_remove(self):
+        disk_spec = mock.Mock()
+        self.session.vim.client.factory.create.return_value = disk_spec
+
+        disk_device = mock.sentinel.disk_device
+        self.vops._create_spec_for_disk_remove(disk_device)
+
+        self.session.vim.client.factory.create.assert_called_once_with(
+            'ns0:VirtualDeviceConfigSpec')
+        self.assertEqual('remove', disk_spec.operation)
+        self.assertEqual(disk_device, disk_spec.device)
+
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                '_create_spec_for_disk_remove')
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                '_reconfigure_backing')
+    def test_detach_disk_from_backing(self, reconfigure_backing, create_spec):
+        disk_spec = mock.sentinel.disk_spec
+        create_spec.return_value = disk_spec
+
+        reconfig_spec = mock.Mock()
+        self.session.vim.client.factory.create.return_value = reconfig_spec
+
+        backing = mock.sentinel.backing
+        disk_device = mock.sentinel.disk_device
+        self.vops.detach_disk_from_backing(backing, disk_device)
+
+        create_spec.assert_called_once_with(disk_device)
+        self.session.vim.client.factory.create.assert_called_once_with(
+            'ns0:VirtualMachineConfigSpec')
+        self.assertEqual([disk_spec], reconfig_spec.deviceChange)
+        reconfigure_backing.assert_called_once_with(backing, reconfig_spec)
+
     def test_rename_backing(self):
         task = mock.sentinel.task
         self.session.invoke_api.return_value = task
@@ -1653,6 +1670,31 @@ class VolumeOpsTestCase(test.TestCase):
                                            force=True)
         self.session.wait_for_task.assert_called_once_with(task)
 
+    def test_move_vmdk_file(self):
+        task = mock.sentinel.task
+        invoke_api = self.session.invoke_api
+        invoke_api.return_value = task
+
+        disk_mgr = self.session.vim.service_content.virtualDiskManager
+        src_dc_ref = mock.sentinel.src_dc_ref
+        src_vmdk_file_path = mock.sentinel.src_vmdk_file_path
+        dest_dc_ref = mock.sentinel.dest_dc_ref
+        dest_vmdk_file_path = mock.sentinel.dest_vmdk_file_path
+        self.vops.move_vmdk_file(src_dc_ref,
+                                 src_vmdk_file_path,
+                                 dest_vmdk_file_path,
+                                 dest_dc_ref=dest_dc_ref)
+
+        invoke_api.assert_called_once_with(self.session.vim,
+                                           'MoveVirtualDisk_Task',
+                                           disk_mgr,
+                                           sourceName=src_vmdk_file_path,
+                                           sourceDatacenter=src_dc_ref,
+                                           destName=dest_vmdk_file_path,
+                                           destDatacenter=dest_dc_ref,
+                                           force=True)
+        self.session.wait_for_task.assert_called_once_with(task)
+
     def test_delete_vmdk_file(self):
         task = mock.sentinel.task
         invoke_api = self.session.invoke_api
@@ -1791,6 +1833,53 @@ class VolumeOpsTestCase(test.TestCase):
             'ClusterComputeResource', self.MAX_OBJECTS)
         continue_retrieval.assert_called_once_with(retrieve_result)
 
+    def test_get_entity_by_inventory_path(self):
+        self.session.invoke_api.return_value = mock.sentinel.ref
+
+        path = mock.sentinel.path
+        ret = self.vops.get_entity_by_inventory_path(path)
+        self.assertEqual(mock.sentinel.ref, ret)
+        self.session.invoke_api.assert_called_once_with(
+            self.session.vim,
+            "FindByInventoryPath",
+            self.session.vim.service_content.searchIndex,
+            inventoryPath=path)
+
+    def test_get_disk_devices(self):
+        disk_device = mock.Mock()
+        disk_device.__class__.__name__ = 'VirtualDisk'
+
+        controller_device = mock.Mock()
+        controller_device.__class__.__name__ = 'VirtualLSILogicController'
+
+        devices = mock.Mock()
+        devices.__class__.__name__ = "ArrayOfVirtualDevice"
+        devices.VirtualDevice = [disk_device, controller_device]
+        self.session.invoke_api.return_value = devices
+
+        vm = mock.sentinel.vm
+        self.assertEqual([disk_device], self.vops._get_disk_devices(vm))
+        self.session.invoke_api.assert_called_once_with(
+            vim_util, 'get_object_property', self.session.vim,
+            vm, 'config.hardware.device')
+
+    def _create_disk_device(self, file_name):
+        backing = mock.Mock(fileName=file_name)
+        backing.__class__.__name__ = 'VirtualDiskFlatVer2BackingInfo'
+        return mock.Mock(backing=backing)
+
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                '_get_disk_devices')
+    def test_get_disk_device(self, get_disk_devices):
+        dev_1 = self._create_disk_device('[ds1] foo/foo.vmdk')
+        dev_2 = self._create_disk_device('[ds1] foo/foo_1.vmdk')
+        get_disk_devices.return_value = [dev_1, dev_2]
+
+        vm = mock.sentinel.vm
+        self.assertEqual(dev_2,
+                         self.vops.get_disk_device(vm, '[ds1] foo/foo_1.vmdk'))
+        get_disk_devices.assert_called_once_with(vm)
+
 
 class VirtualDiskPathTest(test.TestCase):
     """Unit tests for VirtualDiskPath."""
