@@ -24,6 +24,8 @@ import oslo_versionedobjects
 from cinder import context
 from cinder import db
 from cinder import objects
+from cinder.objects import fields
+from cinder.tests.unit import fake_constants as fake
 
 
 def get_test_admin_context():
@@ -52,7 +54,8 @@ def create_volume(ctxt,
     vol['user_id'] = ctxt.user_id
     vol['project_id'] = ctxt.project_id
     vol['status'] = status
-    vol['migration_status'] = migration_status
+    if migration_status:
+        vol['migration_status'] = migration_status
     vol['display_name'] = display_name
     vol['display_description'] = display_description
     vol['attach_status'] = 'detached'
@@ -64,11 +67,16 @@ def create_volume(ctxt,
     for key in kwargs:
         vol[key] = kwargs[key]
     vol['replication_status'] = replication_status
-    vol['replication_extended_status'] = replication_extended_status
-    vol['replication_driver_data'] = replication_driver_data
-    vol['previous_status'] = previous_status
+    if replication_extended_status:
+        vol['replication_extended_status'] = replication_extended_status
+    if replication_driver_data:
+        vol['replication_driver_data'] = replication_driver_data
+    if previous_status:
+        vol['previous_status'] = previous_status
 
-    return db.volume_create(ctxt, vol)
+    volume = objects.Volume(ctxt, **vol)
+    volume.create()
+    return volume
 
 
 def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
@@ -96,8 +104,8 @@ def create_snapshot(ctxt,
     vol = db.volume_get(ctxt, volume_id)
     snap = objects.Snapshot(ctxt)
     snap.volume_id = volume_id
-    snap.user_id = ctxt.user_id or 'fake_user_id'
-    snap.project_id = ctxt.project_id or 'fake_project_id'
+    snap.user_id = ctxt.user_id or fake.user_id
+    snap.project_id = ctxt.project_id or fake.project_id
     snap.status = status
     snap.volume_size = vol['size']
     snap.display_name = display_name
@@ -111,7 +119,7 @@ def create_consistencygroup(ctxt,
                             host='test_host@fakedrv#fakepool',
                             name='test_cg',
                             description='this is a test cg',
-                            status='available',
+                            status=fields.ConsistencyGroupStatus.AVAILABLE,
                             availability_zone='fake_az',
                             volume_type_id=None,
                             cgsnapshot_id=None,
@@ -121,8 +129,8 @@ def create_consistencygroup(ctxt,
 
     cg = objects.ConsistencyGroup(ctxt)
     cg.host = host
-    cg.user_id = ctxt.user_id or 'fake_user_id'
-    cg.project_id = ctxt.project_id or 'fake_project_id'
+    cg.user_id = ctxt.user_id or fake.user_id
+    cg.project_id = ctxt.project_id or fake.project_id
     cg.status = status
     cg.name = name
     cg.description = description
@@ -139,32 +147,35 @@ def create_consistencygroup(ctxt,
 
 
 def create_cgsnapshot(ctxt,
-                      name='test_cgsnap',
-                      description='this is a test cgsnap',
-                      status='available',
-                      consistencygroup_id=None,
+                      consistencygroup_id,
+                      name='test_cgsnapshot',
+                      description='this is a test cgsnapshot',
+                      status='creating',
                       **kwargs):
     """Create a cgsnapshot object in the DB."""
-    cgsnap = {}
-    cgsnap['user_id'] = ctxt.user_id
-    cgsnap['project_id'] = ctxt.project_id
-    cgsnap['status'] = status
-    cgsnap['name'] = name
-    cgsnap['description'] = description
-    cgsnap['consistencygroup_id'] = consistencygroup_id
+    cgsnap = objects.CGSnapshot(ctxt)
+    cgsnap.user_id = ctxt.user_id or fake.user_id
+    cgsnap.project_id = ctxt.project_id or fake.project_id
+    cgsnap.status = status
+    cgsnap.name = name
+    cgsnap.description = description
+    cgsnap.consistencygroup_id = consistencygroup_id
     for key in kwargs:
-        cgsnap[key] = kwargs[key]
-    return db.cgsnapshot_create(ctxt, cgsnap)
+        setattr(cgsnap, key, kwargs[key])
+    cgsnap.create()
+    return cgsnap
 
 
 def create_backup(ctxt,
                   volume_id,
                   display_name='test_backup',
                   display_description='This is a test backup',
-                  status='creating',
+                  status=fields.BackupStatus.CREATING,
                   parent_id=None,
                   temp_volume_id=None,
-                  temp_snapshot_id=None):
+                  temp_snapshot_id=None,
+                  snapshot_id=None,
+                  data_timestamp=None):
     backup = {}
     backup['volume_id'] = volume_id
     backup['user_id'] = ctxt.user_id
@@ -182,6 +193,8 @@ def create_backup(ctxt,
     backup['object_count'] = 22
     backup['temp_volume_id'] = temp_volume_id
     backup['temp_snapshot_id'] = temp_snapshot_id
+    backup['snapshot_id'] = snapshot_id
+    backup['data_timestamp'] = data_timestamp
     return db.backup_create(ctxt, backup)
 
 
@@ -233,3 +246,18 @@ def get_file_spec():
                 set(dir(_io.BytesIO))))
         else:
             file_spec = file
+
+
+def generate_timeout_series(timeout):
+    """Generate a series of times that exceeds the given timeout.
+
+    Yields a series of fake time.time() floating point numbers
+    such that the difference between each pair in the series just
+    exceeds the timeout value that is passed in.  Useful for
+    mocking time.time() in methods that otherwise wait for timeout
+    seconds.
+    """
+    iteration = 0
+    while True:
+        iteration += 1
+        yield (iteration * timeout) + iteration
