@@ -1,5 +1,6 @@
 # Copyright (c) - 2014, Clinton Knight.  All rights reserved.
 # Copyright (c) - 2015, Tom Barron.  All rights reserved.
+# Copyright (c) - 2016 Chuck Fouts. All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -101,7 +102,6 @@ FC_FABRIC_MAP = {'fabricB':
 FC_TARGET_INFO = {'driver_volume_type': 'fibre_channel',
                   'data': {'target_lun': 1,
                            'initiator_target_map': FC_I_T_MAP,
-                           'access_mode': 'rw',
                            'target_wwn': FC_TARGET_WWPNS,
                            'target_discovered': True}}
 
@@ -131,9 +131,12 @@ ISCSI_SERVICE_IQN = 'fake_iscsi_service_iqn'
 
 ISCSI_CONNECTION_PROPERTIES = {
     'data': {
-        'auth_method': 'fake',
+        'auth_method': 'fake_method',
         'auth_password': 'auth',
         'auth_username': 'provider',
+        'discovery_auth_method': 'fake_method',
+        'discovery_auth_username': 'provider',
+        'discovery_auth_password': 'auth',
         'target_discovered': False,
         'target_iqn': ISCSI_SERVICE_IQN,
         'target_lun': 42,
@@ -207,10 +210,14 @@ CLONE_DESTINATION = {
     'id': CLONE_DESTINATION_ID,
 }
 
+SNAPSHOT_NAME = 'fake_snapshot_name'
+SNAPSHOT_LUN_HANDLE = 'fake_snapshot_lun_handle'
+
 SNAPSHOT = {
-    'name': 'fake_snapshot_name',
+    'name': SNAPSHOT_NAME,
     'volume_size': SIZE,
     'volume_id': 'fake_volume_id',
+    'busy': False,
 }
 
 VOLUME_REF = {'name': 'fake_vref_name', 'size': 42}
@@ -218,6 +225,7 @@ VOLUME_REF = {'name': 'fake_vref_name', 'size': 42}
 FAKE_CMODE_POOLS = [
     {
         'QoS_support': True,
+        'consistencygroup_support': True,
         'free_capacity_gb': 3.72,
         'netapp_compression': u'true',
         'netapp_dedup': u'true',
@@ -232,10 +240,13 @@ FAKE_CMODE_POOLS = [
         'pool_name': 'open123',
         'reserved_percentage': 0,
         'total_capacity_gb': 4.65,
-        'thin_provisioned_support': True,
-        'thick_provisioned_support': False,
+        'thin_provisioning_support': True,
+        'thick_provisioning_support': False,
         'provisioned_capacity_gb': 0.93,
         'max_over_subscription_ratio': 20.0,
+        'utilization': 30.0,
+        'filter_function': 'filter',
+        'goodness_function': 'goodness',
     }
 ]
 
@@ -330,17 +341,89 @@ FAKE_7MODE_VOL1 = [netapp_api.NaElement(
 FAKE_7MODE_POOLS = [
     {
         'pool_name': 'open123',
+        'consistencygroup_support': True,
         'QoS_support': False,
         'reserved_percentage': 0,
         'total_capacity_gb': 0.0,
         'free_capacity_gb': 0.0,
         'max_over_subscription_ratio': 20.0,
-        'thin_provisioned_support': False,
-        'thick_provisioned_support': True,
+        'thin_provisioning_support': False,
+        'thick_provisioning_support': True,
         'provisioned_capacity_gb': 0.0,
+        'utilization': 30.0,
+        'filter_function': 'filter',
+        'goodness_function': 'goodness',
     }
 ]
 
+CG_VOLUME_NAME = 'fake_cg_volume'
+CG_GROUP_NAME = 'fake_consistency_group'
+SOURCE_CG_VOLUME_NAME = 'fake_source_cg_volume'
+CG_VOLUME_ID = 'fake_cg_volume_id'
+CG_VOLUME_SIZE = 100
+SOURCE_CG_VOLUME_ID = 'fake_source_cg_volume_id'
+CONSISTENCY_GROUP_NAME = 'fake_cg'
+SOURCE_CONSISTENCY_GROUP_ID = 'fake_source_cg_id'
+CONSISTENCY_GROUP_ID = 'fake_cg_id'
+CG_SNAPSHOT_ID = 'fake_cg_snapshot_id'
+CG_SNAPSHOT_NAME = 'snapshot-' + CG_SNAPSHOT_ID
+CG_VOLUME_SNAPSHOT_ID = 'fake_cg_volume_snapshot_id'
+
+CG_LUN_METADATA = {
+    'OsType': None,
+    'Path': '/vol/aggr1/fake_cg_volume',
+    'SpaceReserved': 'true',
+    'Qtree': None,
+    'Volume': POOL_NAME,
+}
+
+SOURCE_CG_VOLUME = {
+    'name': SOURCE_CG_VOLUME_NAME,
+    'size': CG_VOLUME_SIZE,
+    'id': SOURCE_CG_VOLUME_ID,
+    'host': 'hostname@backend#cdot',
+    'consistencygroup_id': None,
+    'status': 'fake_status',
+}
+
+CG_VOLUME = {
+    'name': CG_VOLUME_NAME,
+    'size': 100,
+    'id': CG_VOLUME_ID,
+    'host': 'hostname@backend#cdot',
+    'consistencygroup_id': CONSISTENCY_GROUP_ID,
+    'status': 'fake_status',
+}
+
+SOURCE_CONSISTENCY_GROUP = {
+    'id': SOURCE_CONSISTENCY_GROUP_ID,
+    'status': 'fake_status',
+}
+
+CONSISTENCY_GROUP = {
+    'id': CONSISTENCY_GROUP_ID,
+    'status': 'fake_status',
+    'name': CG_GROUP_NAME,
+}
+
+CG_SNAPSHOT = {
+    'id': CG_SNAPSHOT_ID,
+    'name': CG_SNAPSHOT_NAME,
+    'volume_size': CG_VOLUME_SIZE,
+    'consistencygroup_id': CONSISTENCY_GROUP_ID,
+    'status': 'fake_status',
+    'volume_id': 'fake_source_volume_id',
+}
+
+CG_VOLUME_SNAPSHOT = {
+    'name': CG_SNAPSHOT_NAME,
+    'volume_size': CG_VOLUME_SIZE,
+    'cgsnapshot_id': CG_SNAPSHOT_ID,
+    'id': CG_VOLUME_SNAPSHOT_ID,
+    'status': 'fake_status',
+    'volume_id': CG_VOLUME_ID,
+}
+
 
 class test_volume(object):
     pass
