@@ -24,7 +24,38 @@ import mock
 from cinder.volume import configuration as conf
 from cinder.volume.drivers.netapp.eseries import utils
 import cinder.volume.drivers.netapp.options as na_opts
+import cinder.volume.drivers.netapp.utils as na_utils
 
+FAKE_CINDER_VOLUME = {
+    'id': '114774fb-e15a-4fae-8ee2-c9723e3645ef',
+    'size': 1,
+    'volume_name': 'lun1',
+    'host': 'hostname@backend#DDP',
+    'os_type': 'linux',
+    'provider_location': 'lun1',
+    'name_id': '114774fb-e15a-4fae-8ee2-c9723e3645ef',
+    'provider_auth': 'provider a b',
+    'project_id': 'project',
+    'display_name': None,
+    'display_description': 'lun1',
+    'volume_type_id': None,
+    'migration_status': None,
+    'attach_status': "detached"
+}
+
+FAKE_CINDER_SNAPSHOT = {
+    'id': '78f95b9d-3f02-4781-a512-1a1c921d48a1',
+    'volume': FAKE_CINDER_VOLUME
+}
+
+FAKE_CINDER_CG = {
+    'id': '78f95b9d-3f02-4781-a512-1a1c951d48a2',
+}
+
+FAKE_CINDER_CG_SNAPSHOT = {
+    'id': '78f95b9d-4d13-4781-a512-1a1c951d6a6',
+    'consistencygroup_id': FAKE_CINDER_CG['id'],
+}
 
 MULTIATTACH_HOST_GROUP = {
     'clusterRef': '8500000060080E500023C7340036035F515B78FC',
@@ -36,6 +67,8 @@ FOREIGN_HOST_GROUP = {
     'label': 'FOREIGN HOST GROUP',
 }
 
+HOST_GROUPS = [MULTIATTACH_HOST_GROUP, FOREIGN_HOST_GROUP]
+
 SSC_POOLS = [
     {
         "poolId": "0400000060080E5000290D8000009C9955828DD2",
@@ -471,6 +504,7 @@ VOLUME = VOLUMES[0]
 
 STORAGE_POOL = {
     'label': 'DDP',
+    'id': 'fakevolgroupref',
     'volumeGroupRef': 'fakevolgroupref',
     'raidLevel': 'raidDiskPool',
     'usedSpace': '16413217521664',
@@ -633,15 +667,15 @@ STORAGE_SYSTEM = {
 }
 
 SNAPSHOT_GROUP = {
+    'id': '3300000060080E500023C7340000098D5294AC9A',
     'status': 'optimal',
     'autoDeleteLimit': 0,
     'maxRepositoryCapacity': '-65536',
     'rollbackStatus': 'none',
     'unusableRepositoryCapacity': '0',
-    'pitGroupRef':
-    '3300000060080E500023C7340000098D5294AC9A',
+    'pitGroupRef': '3300000060080E500023C7340000098D5294AC9A',
     'clusterSize': 65536,
-    'label': 'C6JICISVHNG2TFZX4XB5ZWL7O',
+    'label': 'C6JICISVHNG2TFZX4XB5ZWL7F',
     'maxBaseCapacity': '476187142128128',
     'repositoryVolume': '3600000060080E500023BB3400001FA952CEF12C',
     'fullWarnThreshold': 99,
@@ -652,10 +686,13 @@ SNAPSHOT_GROUP = {
     'consistencyGroupRef': '0000000000000000000000000000000000000000',
     'volumeHandle': 49153,
     'consistencyGroup': False,
-    'baseVolume': '0200000060080E500023C734000009825294A534'
+    'baseVolume': '0200000060080E500023C734000009825294A534',
+    'snapshotCount': 32
 }
 
 SNAPSHOT_IMAGE = {
+    'id': '3400000060080E500023BB3400631F335294A5A8',
+    'baseVol': '0200000060080E500023C734000009825294A534',
     'status': 'optimal',
     'pitCapacity': '2147483648',
     'pitTimestamp': '1389315375',
@@ -665,7 +702,86 @@ SNAPSHOT_IMAGE = {
     'activeCOW': True,
     'isRollbackSource': False,
     'pitRef': '3400000060080E500023BB3400631F335294A5A8',
-    'pitSequenceNumber': '19'
+    'pitSequenceNumber': '19',
+    'consistencyGroupId': '0000000000000000000000000000000000000000',
+}
+
+SNAPSHOT_VOLUME = {
+    'id': '35000000600A0980006077F80000F8BF566581AA',
+    'viewRef': '35000000600A0980006077F80000F8BF566581AA',
+    'worldWideName': '600A0980006077F80000F8BF566581AA',
+    'baseVol': '02000000600A0980006077F80000F89B56657E26',
+    'basePIT': '0000000000000000000000000000000000000000',
+    'boundToPIT': False,
+    'accessMode': 'readOnly',
+    'label': 'UZJ45SLUKNGWRF3QZHBTOG4C4E_DEL',
+    'status': 'stopped',
+    'currentManager': '070000000000000000000001',
+    'preferredManager': '070000000000000000000001',
+    'repositoryVolume': '0000000000000000000000000000000000000000',
+    'fullWarnThreshold': 0,
+    'viewTime': '1449453419',
+    'viewSequenceNumber': '2104',
+    'volumeHandle': 16510,
+    'clusterSize': 0,
+    'maxRepositoryCapacity': '0',
+    'unusableRepositoryCapacity': '0',
+    'membership': {
+        'viewType': 'individual',
+        'cgViewRef': None
+    },
+    'mgmtClientAttribute': 0,
+    'offline': False,
+    'volumeFull': False,
+    'repositoryCapacity': '0',
+    'baseVolumeCapacity': '1073741824',
+    'totalSizeInBytes': '0',
+    'consistencyGroupId': None,
+    'volumeCopyTarget': False,
+    'cloneCopy': False,
+    'volumeCopySource': False,
+    'pitBaseVolume': False,
+    'asyncMirrorTarget': False,
+    'asyncMirrorSource': False,
+    'protectionType': 'type0Protection',
+    'remoteMirrorSource': False,
+    'remoteMirrorTarget': False,
+    'wwn': '600A0980006077F80000F8BF566581AA',
+    'listOfMappings': [],
+    'mapped': False,
+    'currentControllerId': '070000000000000000000001',
+    'preferredControllerId': '070000000000000000000001',
+    'onlineVolumeCopy': False,
+    'objectType': 'pitView',
+    'name': 'UZJ45SLUKNGWRF3QZHBTOG4C4E',
+}
+
+FAKE_BACKEND_STORE = {
+    'key': 'cinder-snapshots',
+    'value': '{"3300000060080E50003416400000E90D56B047E5":"2"}'
+}
+
+FAKE_CINDER_VOLUME = {
+    'id': '114774fb-e15a-4fae-8ee2-c9723e3645ef',
+    'size': 1,
+    'volume_name': 'lun1',
+    'host': 'hostname@backend#DDP',
+    'os_type': 'linux',
+    'provider_location': 'lun1',
+    'name_id': '114774fb-e15a-4fae-8ee2-c9723e3645ef',
+    'provider_auth': 'provider a b',
+    'project_id': 'project',
+    'display_name': None,
+    'display_description': 'lun1',
+    'volume_type_id': None,
+    'migration_status': None,
+    'attach_status': "detached"
+}
+
+FAKE_CINDER_SNAPSHOT = {
+    'id': '78f95b9d-3f02-4781-a512-1a1c921d48a1',
+    'volume': FAKE_CINDER_VOLUME,
+    'provider_id': '3400000060080E500023BB3400631F335294A5A8'
 }
 
 HARDWARE_INVENTORY_SINGLE_CONTROLLER = {
@@ -905,6 +1021,55 @@ FAKE_CLIENT_PARAMS = {
     'password': 'rw',
 }
 
+FAKE_CONSISTENCY_GROUP = {
+    'cgRef': '2A000000600A0980006077F8008702F45480F41A',
+    'label': '5BO5GPO4PFGRPMQWEXGTILSAUI',
+    'repFullPolicy': 'failbasewrites',
+    'fullWarnThreshold': 75,
+    'autoDeleteLimit': 0,
+    'rollbackPriority': 'medium',
+    'uniqueSequenceNumber': [8940, 8941, 8942],
+    'creationPendingStatus': 'none',
+    'name': '5BO5GPO4PFGRPMQWEXGTILSAUI',
+    'id': '2A000000600A0980006077F8008702F45480F41A'
+}
+
+FAKE_CONSISTENCY_GROUP_MEMBER = {
+    'consistencyGroupId': '2A000000600A0980006077F8008702F45480F41A',
+    'volumeId': '02000000600A0980006077F8000002F55480F421',
+    'volumeWwn': '600A0980006077F8000002F55480F421',
+    'baseVolumeName': 'I5BHHNILUJGZHEUD4S36GCOQYA',
+    'clusterSize': 65536,
+    'totalRepositoryVolumes': 1,
+    'totalRepositoryCapacity': '4294967296',
+    'usedRepositoryCapacity': '5636096',
+    'fullWarnThreshold': 75,
+    'totalSnapshotImages': 3,
+    'totalSnapshotVolumes': 2,
+    'autoDeleteSnapshots': False,
+    'autoDeleteLimit': 0,
+    'pitGroupId': '33000000600A0980006077F8000002F85480F435',
+    'repositoryVolume': '36000000600A0980006077F8000002F75480F435'
+}
+FAKE_CONSISTENCY_GROUP_SNAPSHOT_VOLUME = {
+    'id': '2C00000060080E500034194F002C96A256BD50F9',
+    'name': '6TRZHKDG75DVLBC2JU5J647RME',
+    'cgViewRef': '2C00000060080E500034194F002C96A256BD50F9',
+    'groupRef': '2A00000060080E500034194F0087969856BD2D67',
+    'label': '6TRZHKDG75DVLBC2JU5J647RME',
+    'viewTime': '1455221060',
+    'viewSequenceNumber': '10',
+}
+
+
+def list_snapshot_groups(numGroups):
+    snapshots = []
+    for n in range(0, numGroups):
+        s = copy.deepcopy(SNAPSHOT_GROUP)
+        s['label'] = s['label'][:-1] + str(n)
+        snapshots.append(s)
+    return snapshots
+
 
 def create_configuration_eseries():
     config = conf.Configuration(None)
@@ -955,9 +1120,13 @@ def deepcopy_return_value_class_decorator(cls):
 
 @deepcopy_return_value_class_decorator
 class FakeEseriesClient(object):
+    features = na_utils.Features()
 
     def __init__(self, *args, **kwargs):
-        pass
+        self.features.add_feature('AUTOSUPPORT')
+        self.features.add_feature('SSC_API_V2')
+        self.features.add_feature('REST_1_3_RELEASE')
+        self.features.add_feature('REST_1_4_RELEASE')
 
     def list_storage_pools(self):
         return STORAGE_POOLS
@@ -1005,7 +1174,7 @@ class FakeEseriesClient(object):
         return MULTIATTACH_HOST_GROUP
 
     def list_host_groups(self):
-        return [MULTIATTACH_HOST_GROUP]
+        return [MULTIATTACH_HOST_GROUP, FOREIGN_HOST_GROUP]
 
     def get_host_group_by_name(self, name, *args, **kwargs):
         host_groups = self.list_host_groups()
@@ -1024,6 +1193,9 @@ class FakeEseriesClient(object):
     def get_host(self, *args, **kwargs):
         return HOST
 
+    def create_volume(self, *args, **kwargs):
+        return VOLUME
+
     def create_volume_mapping(self, *args, **kwargs):
         return VOLUME_MAPPING
 
@@ -1057,6 +1229,12 @@ class FakeEseriesClient(object):
     def list_snapshot_images(self):
         return [SNAPSHOT_IMAGE]
 
+    def list_snapshot_image(self, *args, **kwargs):
+        return SNAPSHOT_IMAGE
+
+    def create_cg_snapshot_view(self, *args, **kwargs):
+        return SNAPSHOT_VOLUME
+
     def list_host_types(self):
         return [
             {
@@ -1085,14 +1263,15 @@ class FakeEseriesClient(object):
     def get_eseries_api_info(self, verify=False):
         return 'Proxy', '1.53.9010.0005'
 
-    def set_counter(self, key):
+    def set_counter(self, key, value):
         pass
 
     def add_autosupport_data(self, *args):
         pass
 
     def get_serial_numbers(self):
-        pass
+        return FAKE_ASUP_DATA.get('controller1-serial'), FAKE_ASUP_DATA.get(
+            'controller2-serial')
 
     def get_model_name(self):
         pass
@@ -1101,7 +1280,7 @@ class FakeEseriesClient(object):
         pass
 
     def get_firmware_version(self):
-        return FAKE_POST_INVOKE_DATA["system-version"]
+        return FAKE_ASUP_DATA['system-version']
 
     def create_volume_copy_job(self, *args, **kwargs):
         return VOLUME_COPY_JOB
@@ -1112,6 +1291,24 @@ class FakeEseriesClient(object):
     def delete_vol_copy_job(self, *args, **kwargs):
         pass
 
+    def create_snapshot_image(self, *args, **kwargs):
+        return SNAPSHOT_IMAGE
+
+    def create_snapshot_volume(self, *args, **kwargs):
+        return SNAPSHOT_VOLUME
+
+    def list_snapshot_volumes(self, *args, **kwargs):
+        return [SNAPSHOT_VOLUME]
+
+    def list_snapshot_volume(self, *args, **kwargs):
+        return SNAPSHOT_IMAGE
+
+    def create_snapshot_group(self, *args, **kwargs):
+        return SNAPSHOT_GROUP
+
+    def list_snapshot_group(self, *args, **kwargs):
+        return SNAPSHOT_GROUP
+
     def delete_snapshot_volume(self, *args, **kwargs):
         pass
 
@@ -1120,3 +1317,45 @@ class FakeEseriesClient(object):
 
     def update_stored_system_password(self, *args, **kwargs):
         pass
+
+    def update_snapshot_volume(self, *args, **kwargs):
+        return SNAPSHOT_VOLUME
+
+    def delete_snapshot_image(self, *args, **kwargs):
+        pass
+
+    def delete_snapshot_group(self, *args, **kwargs):
+        pass
+
+    def restart_snapshot_volume(self, *args, **kwargs):
+        pass
+
+    def create_consistency_group(self, *args, **kwargs):
+        return FAKE_CONSISTENCY_GROUP
+
+    def delete_consistency_group(self, *args, **kwargs):
+        pass
+
+    def list_consistency_groups(self, *args, **kwargs):
+        return [FAKE_CONSISTENCY_GROUP]
+
+    def remove_consistency_group_member(self, *args, **kwargs):
+        pass
+
+    def add_consistency_group_member(self, *args, **kwargs):
+        pass
+
+    def list_backend_store(self, key):
+        return {}
+
+    def save_backend_store(self, key, val):
+        pass
+
+    def create_consistency_group_snapshot(self, *args, **kwargs):
+        return [SNAPSHOT_IMAGE]
+
+    def get_consistency_group_snapshots(self, *args, **kwargs):
+        return [SNAPSHOT_IMAGE]
+
+    def delete_consistency_group_snapshot(self, *args, **kwargs):
+        pass
