@@ -58,14 +58,17 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
                 White list target ports support
                 Snap copy support
                 Support efficient non-disruptive backup
+        7.0.0 - Clone consistency group support
+                Replication v2 support(managed)
+                Configurable migration rate support
     """
 
     def __init__(self, *args, **kwargs):
-
         super(EMCCLIFCDriver, self).__init__(*args, **kwargs)
         self.cli = emc_vnx_cli.getEMCVnxCli(
             'FC',
-            configuration=self.configuration)
+            configuration=self.configuration,
+            active_backend_id=kwargs.get('active_backend_id'))
         self.VERSION = self.cli.VERSION
 
     def check_for_setup_error(self):
@@ -143,7 +146,6 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
                     'target_discovered': True,
                     'target_lun': 1,
                     'target_wwn': '1234567890123',
-                    'access_mode': 'rw'
                     'initiator_target_map': {
                         '1122334455667788': ['1234567890123']
                     }
@@ -158,7 +160,6 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
                     'target_discovered': True,
                     'target_lun': 1,
                     'target_wwn': ['1234567890123', '0987654321321'],
-                    'access_mode': 'rw'
                     'initiator_target_map': {
                         '1122334455667788': ['1234567890123',
                                              '0987654321321']
@@ -219,7 +220,7 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
             'source-name':<lun name in VNX>
         }
         """
-        self.cli.manage_existing(volume, existing_ref)
+        return self.cli.manage_existing(volume, existing_ref)
 
     def manage_existing_get_size(self, volume, existing_ref):
         """Return size of volume to be managed by manage_existing."""
@@ -229,19 +230,20 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
         """Creates a consistencygroup."""
         return self.cli.create_consistencygroup(context, group)
 
-    def delete_consistencygroup(self, context, group):
+    def delete_consistencygroup(self, context, group, volumes):
         """Deletes a consistency group."""
         return self.cli.delete_consistencygroup(
-            self, context, group)
+            context, group, volumes)
 
-    def create_cgsnapshot(self, context, cgsnapshot):
+    def create_cgsnapshot(self, context, cgsnapshot, snapshots):
         """Creates a cgsnapshot."""
         return self.cli.create_cgsnapshot(
-            self, context, cgsnapshot)
+            context, cgsnapshot, snapshots)
 
-    def delete_cgsnapshot(self, context, cgsnapshot):
+    def delete_cgsnapshot(self, context, cgsnapshot, snapshots):
         """Deletes a cgsnapshot."""
-        return self.cli.delete_cgsnapshot(self, context, cgsnapshot)
+        return self.cli.delete_cgsnapshot(
+            context, cgsnapshot, snapshots)
 
     def get_pool(self, volume):
         """Returns the pool name of a volume."""
@@ -267,7 +269,9 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
                                                          group,
                                                          volumes,
                                                          cgsnapshot,
-                                                         snapshots)
+                                                         snapshots,
+                                                         source_cg,
+                                                         source_vols)
 
     def update_migrated_volume(self, context, volume, new_volume,
                                original_volume_status=None):
@@ -297,3 +301,7 @@ class EMCCLIFCDriver(driver.FibreChannelDriver):
 
     def backup_use_temp_snapshot(self):
         return True
+
+    def failover_host(self, context, volumes, secondary_backend_id):
+        """Failovers volume from primary device to secondary."""
+        return self.cli.failover_host(context, volumes, secondary_backend_id)
