@@ -49,7 +49,7 @@ class HnasBackend(object):
         :param ip0: string IP address of controller
         :param user: string user authentication for array
         :param pw: string password authentication for array
-        :return: formated string with version information
+        :returns: formated string with version information
         """
         LOG.debug('Enable ssh: %s',
                   six.text_type(self.drv_configs['ssh_enabled']))
@@ -127,7 +127,6 @@ class HnasBackend(object):
        :param pw: string password authentication for array
        :returns: formatted string with version information
        """
-
         out, err = self.run_cmd(cmd, ip0, user, pw, "cluster-getmac",
                                 check_exit_code=True)
         hardware = out.split()[2]
@@ -167,7 +166,7 @@ class HnasBackend(object):
         :param ip0: string IP address of controller
         :param user: string user authentication for array
         :param pw: string password authentication for array
-        :return: formated string with iSCSI information
+        :returns: formated string with iSCSI information
         """
 
         out, err = self.run_cmd(cmd, ip0, user, pw,
@@ -195,7 +194,7 @@ class HnasBackend(object):
         :param user: string user authentication for array
         :param pw: string password authentication for array
         :param fslabel: filesystem label we want to get info
-        :return: formated string with filesystems and fsids
+        :returns: formated string with filesystems and fsids
         """
 
         if fslabel is None:
@@ -258,7 +257,7 @@ class HnasBackend(object):
         :param ip0: string IP address of controller
         :param user: string user authentication for array
         :param pw: string password authentication for array
-        :return: EVS id of the file system
+        :returns: EVS id of the file system
         """
 
         out, err = self.run_cmd(cmd, ip0, user, pw, "evsfs", "list",
@@ -363,7 +362,7 @@ class HnasBackend(object):
             tgt_list.append(tgt)
 
         if tgtalias is not None:
-            # We tried to find  'tgtalias' but didn't find. Return a empty
+            # We tried to find  'tgtalias' but didn't find. Return an empty
             # list.
             LOG.debug("There's no target %(alias)s in EVS %(evsid)s.",
                       {'alias': tgtalias, 'evsid': evsid})
@@ -625,7 +624,7 @@ class HnasBackend(object):
         :param evsid: EVSID for the file system
         :param iqn: iSCSI qualified name
         :param hlun: logical unit id
-        :return: formated string
+        :returns: formated string
         """
 
         out, err = self.run_cmd(cmd, ip0, user, pw, "console-context",
@@ -672,7 +671,7 @@ class HnasBackend(object):
         :param targetalias: alias of the target
         :param hdp: data pool of the logical unit
         :param secret: CHAP secret of the target
-        :return: string with full IQN
+        :returns: string with full IQN
         """
 
         _evsid = self.get_evs(cmd, ip0, user, pw, hdp)
@@ -781,8 +780,8 @@ class HnasBackend(object):
         :param pw: string password authentication for array
         :param hdp: pool name used
         :param target_alias: alias of the target
-        :return True if target exists
-        :return list with the target info
+        :returns: True if target exists
+        :returns: list with the target info
         """
 
         LOG.debug("Checking if target %(tgt)s exists.", {'tgt': target_alias})
@@ -808,9 +807,9 @@ class HnasBackend(object):
         :param pw: string password authentication for array
         :param volume_name: number of the LUN
         :param hdp: storage pool of the LUN
-        :return True if the lun is attached
-        :return the LUN id
-        :return Info related to the target
+        :returns: True if the lun is attached
+        :returns: the LUN id
+        :returns: Info related to the target
         """
 
         LOG.debug("Checking if vol %s (hdp: %s) is attached.",
