@@ -2,6 +2,7 @@
 # Copyright (c) 2014 Navneet Singh.  All rights reserved.
 # Copyright (c) 2014 Clinton Knight.  All rights reserved.
 # Copyright (c) 2015 Tom Barron.  All rights reserved.
+# Copyright (c) 2016 Michael Price.  All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -50,10 +51,13 @@ DEPRECATED_SSC_SPECS = {'netapp_unmirrored': 'netapp_mirrored',
                         'netapp_nodedup': 'netapp_dedup',
                         'netapp_nocompression': 'netapp_compression',
                         'netapp_thick_provisioned': 'netapp_thin_provisioned'}
-QOS_KEYS = frozenset(
-    ['maxIOPS', 'total_iops_sec', 'maxBPS', 'total_bytes_sec'])
+QOS_KEYS = frozenset(['maxIOPS', 'maxIOPSperGiB', 'maxBPS', 'maxBPSperGiB'])
 BACKEND_QOS_CONSUMERS = frozenset(['back-end', 'both'])
 
+# Secret length cannot be less than 96 bits. http://tools.ietf.org/html/rfc3723
+CHAP_SECRET_LENGTH = 16
+DEFAULT_CHAP_USER_NAME = 'NetApp_iSCSI_CHAP_Username'
+
 
 def validate_instantiation(**kwargs):
     """Checks if a driver is instantiated other than by the unified driver.
@@ -138,35 +142,35 @@ def round_down(value, precision):
 def log_extra_spec_warnings(extra_specs):
     for spec in (set(extra_specs.keys() if extra_specs else []) &
                  set(OBSOLETE_SSC_SPECS.keys())):
-            LOG.warning(_LW('Extra spec %(old)s is obsolete.  Use %(new)s '
-                            'instead.'), {'old': spec,
-                                          'new': OBSOLETE_SSC_SPECS[spec]})
+        LOG.warning(_LW('Extra spec %(old)s is obsolete.  Use %(new)s '
+                        'instead.'), {'old': spec,
+                                      'new': OBSOLETE_SSC_SPECS[spec]})
     for spec in (set(extra_specs.keys() if extra_specs else []) &
                  set(DEPRECATED_SSC_SPECS.keys())):
-            LOG.warning(_LW('Extra spec %(old)s is deprecated.  Use %(new)s '
-                            'instead.'), {'old': spec,
-                                          'new': DEPRECATED_SSC_SPECS[spec]})
+        LOG.warning(_LW('Extra spec %(old)s is deprecated.  Use %(new)s '
+                        'instead.'), {'old': spec,
+                                      'new': DEPRECATED_SSC_SPECS[spec]})
 
 
 def get_iscsi_connection_properties(lun_id, volume, iqn,
                                     address, port):
 
-        properties = {}
-        properties['target_discovered'] = False
-        properties['target_portal'] = '%s:%s' % (address, port)
-        properties['target_iqn'] = iqn
-        properties['target_lun'] = int(lun_id)
-        properties['volume_id'] = volume['id']
-        auth = volume['provider_auth']
-        if auth:
-            (auth_method, auth_username, auth_secret) = auth.split()
-            properties['auth_method'] = auth_method
-            properties['auth_username'] = auth_username
-            properties['auth_password'] = auth_secret
-        return {
-            'driver_volume_type': 'iscsi',
-            'data': properties,
-        }
+    properties = {}
+    properties['target_discovered'] = False
+    properties['target_portal'] = '%s:%s' % (address, port)
+    properties['target_iqn'] = iqn
+    properties['target_lun'] = int(lun_id)
+    properties['volume_id'] = volume['id']
+    auth = volume['provider_auth']
+    if auth:
+        (auth_method, auth_username, auth_secret) = auth.split()
+        properties['auth_method'] = auth_method
+        properties['auth_username'] = auth_username
+        properties['auth_password'] = auth_secret
+    return {
+        'driver_volume_type': 'iscsi',
+        'data': properties,
+    }
 
 
 def validate_qos_spec(qos_spec):
@@ -200,14 +204,23 @@ def map_qos_spec(qos_spec, volume):
     """Map Cinder QOS spec to limit/throughput-value as used in client API."""
     if qos_spec is None:
         return None
+
     qos_spec = map_dict_to_lower(qos_spec)
     spec = dict(policy_name=get_qos_policy_group_name(volume),
                 max_throughput=None)
-    # IOPS and BPS specifications are exclusive of one another.
-    if 'maxiops' in qos_spec or 'total_iops_sec' in qos_spec:
+
+    # QoS specs are exclusive of one another.
+    if 'maxiops' in qos_spec:
         spec['max_throughput'] = '%siops' % qos_spec['maxiops']
-    elif 'maxbps' in qos_spec or 'total_bytes_sec' in qos_spec:
+    elif 'maxiopspergib' in qos_spec:
+        spec['max_throughput'] = '%siops' % six.text_type(
+            int(qos_spec['maxiopspergib']) * int(volume['size']))
+    elif 'maxbps' in qos_spec:
         spec['max_throughput'] = '%sB/s' % qos_spec['maxbps']
+    elif 'maxbpspergib' in qos_spec:
+        spec['max_throughput'] = '%sB/s' % six.text_type(
+            int(qos_spec['maxbpspergib']) * int(volume['size']))
+
     return spec
 
 
@@ -244,7 +257,7 @@ def get_pool_name_filter_regex(configuration):
 
     :param configuration: The volume driver configuration
     :raise InvalidConfigurationValue: if configured regex pattern is invalid
-    :return: A compiled regex for filtering pool names
+    :returns: A compiled regex for filtering pool names
     """
 
     # If the configuration parameter is specified as an empty string
@@ -487,6 +500,69 @@ class FeatureState(object):
     def __nonzero__(self):
         """Allow a FeatureState object to be tested for truth value
 
-        :return True if the feature is supported, otherwise False
+        :returns: True if the feature is supported, otherwise False
         """
         return self.supported
+
+    def __bool__(self):
+        """py3 Allow a FeatureState object to be tested for truth value
+
+        :returns: True if the feature is supported, otherwise False
+        """
+        return self.supported
+
+
+class BitSet(object):
+    def __init__(self, value=0):
+        self._value = value
+
+    def set(self, bit):
+        self._value |= 1 << bit
+        return self
+
+    def unset(self, bit):
+        self._value &= ~(1 << bit)
+        return self
+
+    def is_set(self, bit):
+        return self._value & 1 << bit
+
+    def __and__(self, other):
+        self._value &= other
+        return self
+
+    def __or__(self, other):
+        self._value |= other
+        return self
+
+    def __invert__(self):
+        self._value = ~self._value
+        return self
+
+    def __xor__(self, other):
+        self._value ^= other
+        return self
+
+    def __lshift__(self, other):
+        self._value <<= other
+        return self
+
+    def __rshift__(self, other):
+        self._value >>= other
+        return self
+
+    def __int__(self):
+        return self._value
+
+    def __str__(self):
+        return bin(self._value)
+
+    def __repr__(self):
+        return str(self._value)
+
+    def __eq__(self, other):
+        return (isinstance(other, self.__class__) and self._value ==
+                other._value) or self._value == int(other)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
