@@ -16,6 +16,7 @@
 import json
 import math
 import random
+import re
 import socket
 import string
 import time
@@ -23,7 +24,6 @@ import warnings
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import importutils
 from oslo_utils import timeutils
 from oslo_utils import units
 import requests
@@ -34,6 +34,7 @@ from cinder import context
 from cinder import exception
 from cinder.i18n import _, _LE, _LW
 from cinder.image import image_utils
+from cinder.objects import fields
 from cinder.volume.drivers.san import san
 from cinder.volume import qos_specs
 from cinder.volume.targets import iscsi as iscsi_driver
@@ -51,12 +52,17 @@ sf_opts = [
                 help='Allow tenants to specify QOS on create'),
 
     cfg.StrOpt('sf_account_prefix',
-               default=None,
                help='Create SolidFire accounts with this prefix. Any string '
                     'can be used here, but the string \"hostname\" is special '
                     'and will create a prefix using the cinder node hostname '
                     '(previous default behavior).  The default is NO prefix.'),
 
+    cfg.StrOpt('sf_volume_prefix',
+               default='UUID-',
+               help='Create SolidFire volumes with this prefix. Volume names '
+                    'are of the form <sf_volume_prefix><cinder-volume-id>.  '
+                    'The default is to use a prefix of \'UUID-\'.'),
+
     cfg.StrOpt('sf_template_account_name',
                default='openstack-vtemplate',
                help='Account name on the SolidFire Cluster to use as owner of '
@@ -69,7 +75,6 @@ sf_opts = [
                      'glance and qemu-conversion on subsequent calls.'),
 
     cfg.StrOpt('sf_svip',
-               default=None,
                help='Overrides default cluster SVIP with the one specified. '
                     'This is required or deployments that have implemented '
                     'the use of VLANs for iSCSI networks in their cloud.'),
@@ -81,16 +86,24 @@ sf_opts = [
                      'memory, very large deployments may want to consider '
                      'setting to False.'),
 
-    cfg.IntOpt('sf_api_port',
-               default=443,
-               min=1, max=65535,
-               help='SolidFire API port. Useful if the device api is behind '
-                    'a proxy on a different port.')]
+    cfg.PortOpt('sf_api_port',
+                default=443,
+                help='SolidFire API port. Useful if the device api is behind '
+                     'a proxy on a different port.'),
 
+    cfg.BoolOpt('sf_enable_vag',
+                default=False,
+                help='Utilize volume access groups on a per-tenant basis.')]
 
 CONF = cfg.CONF
 CONF.register_opts(sf_opts)
 
+# SolidFire API Error Constants
+xExceededLimit = 'xExceededLimit'
+xAlreadyInVolumeAccessGroup = 'xAlreadyInVolumeAccessGroup'
+xVolumeAccessGroupIDDoesNotExist = 'xVolumeAccessGroupIDDoesNotExist'
+xNotInVolumeAccessGroup = 'xNotInVolumeAccessGroup'
+
 
 def retry(exc_tuple, tries=5, delay=1, backoff=2):
     def retry_dec(f):
@@ -177,13 +190,13 @@ class SolidFireDriver(san.SanISCSIDriver):
         if self.configuration.sf_allow_template_caching:
             account = self.configuration.sf_template_account_name
             self.template_account_id = self._create_template_account(account)
-        self.target_driver = (
-            importutils.import_object(
-                'cinder.volume.drivers.solidfire.SolidFireISCSI',
-                solidfire_driver=self,
-                configuration=self.configuration))
+        self.target_driver = SolidFireISCSI(solidfire_driver=self,
+                                            configuration=self.configuration)
         self._set_cluster_uuid()
 
+    def __getattr__(self, attr):
+        return getattr(self.target_driver, attr)
+
     def _set_cluster_uuid(self):
         self.cluster_uuid = (
             self._get_cluster_info()['clusterInfo']['uuid'])
@@ -212,7 +225,7 @@ class SolidFireDriver(san.SanISCSIDriver):
         sf_snaps = self._issue_api_request(
             'ListSnapshots', {}, version='6.0')['result']['snapshots']
         for s in srefs:
-            seek_name = 'UUID-%s' % s['id']
+            seek_name = '%s%s' % (self.configuration.sf_volume_prefix, s['id'])
             sfsnap = next(
                 (ss for ss in sf_snaps if ss['name'] == seek_name), None)
             if sfsnap:
@@ -230,7 +243,7 @@ class SolidFireDriver(san.SanISCSIDriver):
                                           {})['result']['volumes']
         self.volume_map = {}
         for v in vrefs:
-            seek_name = 'UUID-%s' % v['id']
+            seek_name = '%s%s' % (self.configuration.sf_volume_prefix, v['id'])
             sfvol = next(
                 (sv for sv in sf_vols if sv['name'] == seek_name), None)
             if sfvol:
@@ -435,35 +448,14 @@ class SolidFireDriver(san.SanISCSIDriver):
                                             self.cluster_uuid))
         return model_update
 
-    def _do_clone_volume(self, src_uuid,
-                         src_project_id,
-                         vref):
-        """Create a clone of an existing volume or snapshot."""
-
-        attributes = {}
-        qos = {}
-
-        sf_accounts = self._get_sfaccounts_for_tenant(vref['project_id'])
-        if not sf_accounts:
-            sf_account = self._create_sfaccount(vref['project_id'])
-        else:
-            # Check availability for creates
-            sf_account = self._get_account_create_availability(sf_accounts)
-            if not sf_account:
-                # TODO(jdg): We're not doing tertiaries, so fail
-                msg = _('volumes/account exceeded on both primary '
-                        'and secondary SolidFire accounts')
-                raise exception.SolidFireDriverException(msg)
-
-        params = {'name': 'UUID-%s' % vref['id'],
-                  'newAccountID': sf_account['accountID']}
-
+    def _snapshot_discovery(self, src_uuid, params, vref):
         # NOTE(jdg): First check the SF snapshots
         # if we don't find a snap by the given name, just move on to check
         # volumes.  This may be a running system that was updated from
         # before we did snapshots, so need to check both
         is_clone = False
-        snap_name = 'UUID-%s' % src_uuid
+        sf_vol = None
+        snap_name = '%s%s' % (self.configuration.sf_volume_prefix, src_uuid)
         snaps = self._get_sf_snapshots()
         snap = next((s for s in snaps if s["name"] == snap_name), None)
         if snap:
@@ -477,35 +469,46 @@ class SolidFireDriver(san.SanISCSIDriver):
             params['volumeID'] = int(sf_vol['volumeID'])
             params['newSize'] = int(vref['size'] * units.Gi)
             is_clone = True
+        return params, is_clone, sf_vol
 
+    def _do_clone_volume(self, src_uuid,
+                         vref, sf_src_snap=None):
+        """Create a clone of an existing volume or snapshot."""
+
+        attributes = {}
+        sf_account = self._get_create_account(vref['project_id'])
+        params = {'name': '%(prefix)s%(id)s' %
+                  {'prefix': self.configuration.sf_volume_prefix,
+                   'id': vref['id']},
+                  'newAccountID': sf_account['accountID']}
+
+        is_clone = False
+        sf_vol = None
+        if sf_src_snap:
+            # In some scenarios we are passed the snapshot information that we
+            # are supposed to clone.
+            params['snapshotID'] = sf_src_snap['snapshotID']
+            params['volumeID'] = sf_src_snap['volumeID']
+            params['newSize'] = int(vref['size'] * units.Gi)
+        else:
+            params, is_clone, sf_vol = self._snapshot_discovery(src_uuid,
+                                                                params,
+                                                                vref)
         data = self._issue_api_request('CloneVolume', params, version='6.0')
         if (('result' not in data) or ('volumeID' not in data['result'])):
             msg = _("API response: %s") % data
             raise exception.SolidFireAPIException(msg)
 
         sf_volume_id = data['result']['volumeID']
-        if (self.configuration.sf_allow_tenant_qos and
-                vref.get('volume_metadata')is not None):
-            qos = self._set_qos_presets(vref)
-
-        ctxt = context.get_admin_context()
-        type_id = vref.get('volume_type_id', None)
-        if type_id is not None:
-            qos = self._set_qos_by_volume_type(ctxt, type_id)
 
         # NOTE(jdg): all attributes are copied via clone, need to do an update
         # to set any that were provided
         params = {'volumeID': sf_volume_id}
-
         create_time = vref['created_at'].isoformat()
         attributes = {'uuid': vref['id'],
                       'is_clone': 'True',
                       'src_uuid': src_uuid,
                       'created_at': create_time}
-        if qos:
-            params['qos'] = qos
-            for k, v in qos.items():
-                attributes[k] = str(v)
 
         params['attributes'] = attributes
         data = self._issue_api_request('ModifyVolume', params)
@@ -797,6 +800,21 @@ class SolidFireDriver(san.SanISCSIDriver):
             return sfaccount
         return None
 
+    def _get_create_account(self, proj_id):
+        # Retrieve SolidFire accountID to be used for creating volumes.
+        sf_accounts = self._get_sfaccounts_for_tenant(proj_id)
+        if not sf_accounts:
+            sf_account = self._create_sfaccount(proj_id)
+        else:
+            # Check availability for creates
+            sf_account = self._get_account_create_availability(sf_accounts)
+            if not sf_account:
+                # TODO(jdg): We're not doing tertiaries, so fail.
+                msg = _('Volumes/account exceeded on both primary and '
+                        'secondary SolidFire accounts.')
+                raise exception.SolidFireDriverException(msg)
+        return sf_account
+
     def _get_volumes_for_account(self, sf_account_id, cinder_uuid=None):
         # ListVolumesForAccount gives both Active and Deleted
         # we require the solidfire accountID, uuid of volume
@@ -812,6 +830,154 @@ class SolidFireDriver(san.SanISCSIDriver):
         vlist = sorted(vlist, key=lambda k: k['volumeID'])
         return vlist
 
+    def _create_vag(self, iqn, vol_id=None):
+        """Create a volume access group(vag).
+
+           Returns the vag_id.
+        """
+        vag_name = re.sub('[^0-9a-zA-Z]+', '-', iqn)
+        params = {'name': vag_name,
+                  'initiators': [iqn],
+                  'volumes': [vol_id],
+                  'attributes': {'openstack': True}}
+        try:
+            result = self._issue_api_request('CreateVolumeAccessGroup',
+                                             params,
+                                             version='7.0')
+            return result['result']['volumeAccessGroupID']
+        except exception.SolidFireAPIException as error:
+            if xExceededLimit in error.msg:
+                if iqn in error.msg:
+                    # Initiator double registered.
+                    return self._safe_create_vag(iqn, vol_id)
+                else:
+                    # VAG limit reached. Purge and start over.
+                    self._purge_vags()
+                    return self._safe_create_vag(iqn, vol_id)
+            else:
+                raise
+
+    def _safe_create_vag(self, iqn, vol_id=None):
+        # Potential race condition with simultaneous volume attaches to the
+        # same host. To help avoid this, VAG creation makes a best attempt at
+        # finding and using an existing VAG.
+
+        vags = self._get_vags_by_name(iqn)
+        if vags:
+            # Filter through the vags and find the one with matching initiator
+            vag = next((v for v in vags if iqn in v['initiators']), None)
+            if vag:
+                return vag['volumeAccessGroupID']
+            else:
+                # No matches, use the first result, add initiator IQN.
+                vag_id = vags[0]['volumeAccessGroupID']
+                return self._add_initiator_to_vag(iqn, vag_id)
+        return self._create_vag(iqn, vol_id)
+
+    def _base_get_vags(self):
+        params = {}
+        vags = self._issue_api_request(
+            'ListVolumeAccessGroups',
+            params,
+            version='7.0')['result']['volumeAccessGroups']
+        return vags
+
+    def _get_vags_by_name(self, iqn):
+        """Retrieve SolidFire volume access group objects by name.
+
+           Returns an array of vags with a matching name value.
+           Returns an empty array if there are no matches.
+        """
+        vags = self._base_get_vags()
+        vag_name = re.sub('[^0-9a-zA-Z]+', '-', iqn)
+        matching_vags = [vag for vag in vags if vag['name'] == vag_name]
+        return matching_vags
+
+    def _add_initiator_to_vag(self, iqn, vag_id):
+        # Added a vag_id return as there is a chance that we might have to
+        # create a new VAG if our target VAG is deleted underneath us.
+        params = {"initiators": [iqn],
+                  "volumeAccessGroupID": vag_id}
+        try:
+            self._issue_api_request('AddInitiatorsToVolumeAccessGroup',
+                                    params,
+                                    version='7.0')
+            return vag_id
+        except exception.SolidFireAPIException as error:
+            if xAlreadyInVolumeAccessGroup in error.msg:
+                return vag_id
+            elif xVolumeAccessGroupIDDoesNotExist in error.msg:
+                # No locking means sometimes a VAG can be removed by a parallel
+                # volume detach against the same host.
+                return self._safe_create_vag(iqn)
+            else:
+                raise
+
+    def _add_volume_to_vag(self, vol_id, iqn, vag_id):
+        # Added a vag_id return to be consistent with add_initiator_to_vag. It
+        # isn't necessary but may be helpful in the future.
+        params = {"volumeAccessGroupID": vag_id,
+                  "volumes": [vol_id]}
+        try:
+            self._issue_api_request('AddVolumesToVolumeAccessGroup',
+                                    params,
+                                    version='7.0')
+            return vag_id
+
+        except exception.SolidFireAPIException as error:
+            if xAlreadyInVolumeAccessGroup in error.msg:
+                return vag_id
+            elif xVolumeAccessGroupIDDoesNotExist in error.msg:
+                return self._safe_create_vag(iqn, vol_id)
+            else:
+                raise
+
+    def _remove_volume_from_vag(self, vol_id, vag_id):
+        params = {"volumeAccessGroupID": vag_id,
+                  "volumes": [vol_id]}
+        try:
+            self._issue_api_request('RemoveVolumesFromVolumeAccessGroup',
+                                    params,
+                                    version='7.0')
+        except exception.SolidFireAPIException as error:
+            if xNotInVolumeAccessGroup in error.msg:
+                pass
+            elif xVolumeAccessGroupIDDoesNotExist in error.msg:
+                pass
+            else:
+                raise
+
+    def _remove_volume_from_vags(self, vol_id):
+        # Due to all sorts of uncertainty around multiattach, on volume
+        # deletion we make a best attempt at removing the vol_id from VAGs.
+        vags = self._base_get_vags()
+        targets = [v for v in vags if vol_id in v['volumes']]
+        for vag in targets:
+            self._remove_volume_from_vag(vol_id, vag['volumeAccessGroupID'])
+
+    def _remove_vag(self, vag_id):
+        params = {"volumeAccessGroupID": vag_id}
+        try:
+            self._issue_api_request('DeleteVolumeAccessGroup',
+                                    params,
+                                    version='7.0')
+        except exception.SolidFireAPIException as error:
+            if xVolumeAccessGroupIDDoesNotExist not in error.msg:
+                raise
+
+    def _purge_vags(self, limit=10):
+        # Purge up to limit number of VAGs that have no active volumes,
+        # initiators, and an OpenStack attribute. Purge oldest VAGs first.
+        vags = self._base_get_vags()
+        targets = [v for v in vags if v['volumes'] == [] and
+                   v['initiators'] == [] and
+                   v['deletedVolumes'] == [] and
+                   v['attributes'].get('openstack')]
+        sorted_targets = sorted(targets,
+                                key=lambda k: k['volumeAccessGroupID'])
+        for vag in sorted_targets[:limit]:
+            self._remove_vag(vag['volumeAccessGroupID'])
+
     def clone_image(self, context,
                     volume, image_location,
                     image_meta, image_service):
@@ -846,10 +1012,8 @@ class SolidFireDriver(san.SanISCSIDriver):
         except exception.SolidFireAPIException:
             return None, False
 
-        account = self.configuration.sf_template_account_name
         try:
             (data, sfaccount, model) = self._do_clone_volume(image_meta['id'],
-                                                             account,
                                                              volume)
         except exception.VolumeNotFound:
             if self._create_image_volume(context,
@@ -861,11 +1025,22 @@ class SolidFireDriver(san.SanISCSIDriver):
 
             # Ok, should be good to go now, try it again
             (data, sfaccount, model) = self._do_clone_volume(image_meta['id'],
-                                                             account,
                                                              volume)
 
         return model, True
 
+    def _retrieve_qos_setting(self, volume):
+        qos = {}
+        if (self.configuration.sf_allow_tenant_qos and
+                volume.get('volume_metadata')is not None):
+            qos = self._set_qos_presets(volume)
+
+        ctxt = context.get_admin_context()
+        type_id = volume.get('volume_type_id', None)
+        if type_id is not None:
+            qos = self._set_qos_by_volume_type(ctxt, type_id)
+        return qos
+
     def create_volume(self, volume):
         """Create volume on SolidFire device.
 
@@ -882,32 +1057,17 @@ class SolidFireDriver(san.SanISCSIDriver):
         """
         slice_count = 1
         attributes = {}
-        qos = {}
 
-        if (self.configuration.sf_allow_tenant_qos and
-                volume.get('volume_metadata')is not None):
-            qos = self._set_qos_presets(volume)
-
-        ctxt = context.get_admin_context()
-        type_id = volume['volume_type_id']
-        if type_id is not None:
-            qos = self._set_qos_by_volume_type(ctxt, type_id)
+        sf_account = self._get_create_account(volume['project_id'])
+        qos = self._retrieve_qos_setting(volume)
 
         create_time = volume['created_at'].isoformat()
         attributes = {'uuid': volume['id'],
                       'is_clone': 'False',
                       'created_at': create_time}
-        if qos:
-            for k, v in qos.items():
-                attributes[k] = str(v)
 
-        sf_accounts = self._get_sfaccounts_for_tenant(volume['project_id'])
-        if not sf_accounts:
-            sf_account = self._create_sfaccount(volume['project_id'])
-        else:
-            sf_account = self._get_account_create_availability(sf_accounts)
-
-        params = {'name': 'UUID-%s' % volume['id'],
+        vname = '%s%s' % (self.configuration.sf_volume_prefix, volume['id'])
+        params = {'name': vname,
                   'accountID': sf_account['accountID'],
                   'sliceCount': slice_count,
                   'totalSize': int(volume['size'] * units.Gi),
@@ -920,7 +1080,8 @@ class SolidFireDriver(san.SanISCSIDriver):
         migration_status = volume.get('migration_status', None)
         if migration_status and 'target' in migration_status:
             k, v = migration_status.split(':')
-            params['name'] = 'UUID-%s' % v
+            vname = '%s%s' % (self.configuration.sf_volume_prefix, v)
+            params['name'] = vname
             params['attributes']['migration_uuid'] = volume['id']
             params['attributes']['uuid'] = v
         return self._do_volume_create(sf_account, params)
@@ -929,7 +1090,6 @@ class SolidFireDriver(san.SanISCSIDriver):
         """Create a clone of an existing volume."""
         (_data, _sfaccount, model) = self._do_clone_volume(
             src_vref['id'],
-            src_vref['project_id'],
             volume)
 
         return model
@@ -961,6 +1121,8 @@ class SolidFireDriver(san.SanISCSIDriver):
         if sf_vol is not None:
             params = {'volumeID': sf_vol['volumeID']}
             self._issue_api_request('DeleteVolume', params)
+            if volume.get('multiattach'):
+                self._remove_volume_from_vags(sf_vol['volumeID'])
         else:
             LOG.error(_LE("Volume ID %s was not found on "
                           "the SolidFire Cluster while attempting "
@@ -968,7 +1130,8 @@ class SolidFireDriver(san.SanISCSIDriver):
 
     def delete_snapshot(self, snapshot):
         """Delete the specified snapshot from the SolidFire cluster."""
-        sf_snap_name = 'UUID-%s' % snapshot['id']
+        sf_snap_name = '%s%s' % (self.configuration.sf_volume_prefix,
+                                 snapshot['id'])
         accounts = self._get_sfaccounts_for_tenant(snapshot['project_id'])
         snap = None
         for acct in accounts:
@@ -1001,19 +1164,184 @@ class SolidFireDriver(san.SanISCSIDriver):
         if sf_vol is None:
             raise exception.VolumeNotFound(volume_id=snapshot['volume_id'])
         params = {'volumeID': sf_vol['volumeID'],
-                  'name': 'UUID-%s' % snapshot['id']}
-
+                  'name': '%s%s' % (self.configuration.sf_volume_prefix,
+                                    snapshot['id'])}
         return self._do_snapshot_create(params)
 
     def create_volume_from_snapshot(self, volume, snapshot):
         """Create a volume from the specified snapshot."""
         (_data, _sfaccount, model) = self._do_clone_volume(
             snapshot['id'],
-            snapshot['project_id'],
             volume)
 
         return model
 
+    # Consistency group helpers
+    def _create_group_snapshot(self, name, sf_volumes):
+        # Group snapshot is our version of a consistency group snapshot.
+        vol_ids = [vol['volumeID'] for vol in sf_volumes]
+        params = {'name': name,
+                  'volumes': vol_ids}
+        snapshot_id = self._issue_api_request('CreateGroupSnapshot',
+                                              params,
+                                              version='7.0')
+        return snapshot_id['result']
+
+    def _group_snapshot_creator(self, gsnap_name, src_vol_ids):
+        # Common helper that takes in an array of OpenStack Volume UUIDs and
+        # creates a SolidFire group snapshot with them.
+        vol_names = [self.configuration.sf_volume_prefix + vol_id
+                     for vol_id in src_vol_ids]
+        active_sf_vols = self._get_all_active_volumes()
+        target_vols = [vol for vol in active_sf_vols
+                       if vol['name'] in vol_names]
+        if len(src_vol_ids) != len(target_vols):
+            msg = (_("Retrieved a different amount of SolidFire volumes for "
+                     "the provided Cinder volumes. Retrieved: %(ret)s "
+                     "Desired: %(des)s") % {"ret": len(target_vols),
+                                            "des": len(src_vol_ids)})
+            raise exception.SolidFireDriverException(msg)
+
+        result = self._create_group_snapshot(gsnap_name, target_vols)
+        return result
+
+    def _create_temp_group_snapshot(self, source_cg, source_vols):
+        # Take a temporary snapshot to create the volumes for a new
+        # consistency group.
+        gsnap_name = ("%(prefix)s%(id)s-tmp" %
+                      {"prefix": self.configuration.sf_volume_prefix,
+                       "id": source_cg['id']})
+        vol_ids = [vol['id'] for vol in source_vols]
+        self._group_snapshot_creator(gsnap_name, vol_ids)
+        return gsnap_name
+
+    def _list_group_snapshots(self):
+        result = self._issue_api_request('ListGroupSnapshots',
+                                         {},
+                                         version='7.0')
+        return result['result']['groupSnapshots']
+
+    def _get_group_snapshot_by_name(self, name):
+        target_snaps = self._list_group_snapshots()
+        target = next((snap for snap in target_snaps
+                       if snap['name'] == name), None)
+        return target
+
+    def _delete_group_snapshot(self, gsnapid):
+        params = {'groupSnapshotID': gsnapid}
+        self._issue_api_request('DeleteGroupSnapshot',
+                                params,
+                                version='7.0')
+
+    def _delete_cgsnapshot_by_name(self, snap_name):
+        # Common function used to find and delete a snapshot.
+        target = self._get_group_snapshot_by_name(snap_name)
+        if not target:
+            msg = _("Failed to find group snapshot named: %s") % snap_name
+            raise exception.SolidFireDriverException(msg)
+        self._delete_group_snapshot(target['groupSnapshotID'])
+
+    def _find_linked_snapshot(self, target_uuid, group_snap):
+        # Because group snapshots name each individual snapshot the group
+        # snapshot name, we have to trawl through the SolidFire snapshots to
+        # find the SolidFire snapshot from the group that is linked with the
+        # SolidFire volumeID that is linked to the Cinder snapshot source
+        # volume.
+        source_vol = self._get_sf_volume(target_uuid)
+        target_snap = next((sn for sn in group_snap['members']
+                            if sn['volumeID'] == source_vol['volumeID']), None)
+        return target_snap
+
+    def _create_clone_from_sf_snapshot(self, target_uuid, src_uuid,
+                                       sf_group_snap, vol):
+        # Find the correct SolidFire backing snapshot.
+        sf_src_snap = self._find_linked_snapshot(target_uuid,
+                                                 sf_group_snap)
+        _data, _sfaccount, model = self._do_clone_volume(src_uuid,
+                                                         vol,
+                                                         sf_src_snap)
+        model['id'] = vol['id']
+        model['status'] = 'available'
+        return model
+
+    # Required consistency group functions
+    def create_consistencygroup(self, ctxt, group):
+        # SolidFire does not have a viable means for storing consistency group
+        # volume associations. So, we're just going to play along with the
+        # consistency group song and dance. There will be a lot of no-ops
+        # because of this.
+        return {'status': fields.ConsistencyGroupStatus.AVAILABLE}
+
+    def create_consistencygroup_from_src(self, ctxt, group, volumes,
+                                         cgsnapshot, snapshots,
+                                         source_cg, source_vols):
+        if cgsnapshot and snapshots:
+            sf_name = self.configuration.sf_volume_prefix + cgsnapshot['id']
+            sf_group_snap = self._get_group_snapshot_by_name(sf_name)
+
+            # Go about creating volumes from provided snaps.
+            vol_models = []
+            for vol, snap in zip(volumes, snapshots):
+                vol_models.append(self._create_clone_from_sf_snapshot(
+                    snap['volume_id'],
+                    snap['id'],
+                    sf_group_snap,
+                    vol))
+            return ({'status': fields.ConsistencyGroupStatus.AVAILABLE},
+                    vol_models)
+
+        elif source_cg and source_vols:
+            # Create temporary group snapshot.
+            gsnap_name = self._create_temp_group_snapshot(source_cg,
+                                                          source_vols)
+            try:
+                sf_group_snap = self._get_group_snapshot_by_name(gsnap_name)
+                # For each temporary snapshot clone the volume.
+                vol_models = []
+                for vol in volumes:
+                    vol_models.append(self._create_clone_from_sf_snapshot(
+                        vol['source_volid'],
+                        vol['source_volid'],
+                        sf_group_snap,
+                        vol))
+            finally:
+                self._delete_cgsnapshot_by_name(gsnap_name)
+            return {'status': 'available'}, vol_models
+
+    def create_cgsnapshot(self, ctxt, cgsnapshot, snapshots):
+        vol_ids = [snapshot['volume_id'] for snapshot in snapshots]
+        vol_names = [self.configuration.sf_volume_prefix + vol_id
+                     for vol_id in vol_ids]
+        active_sf_vols = self._get_all_active_volumes()
+        target_vols = [vol for vol in active_sf_vols
+                       if vol['name'] in vol_names]
+        if len(snapshots) != len(target_vols):
+            msg = (_("Retrieved a different amount of SolidFire volumes for "
+                     "the provided Cinder snapshots. Retrieved: %(ret)s "
+                     "Desired: %(des)s") % {"ret": len(target_vols),
+                                            "des": len(snapshots)})
+            raise exception.SolidFireDriverException(msg)
+        snap_name = self.configuration.sf_volume_prefix + cgsnapshot['id']
+        self._create_group_snapshot(snap_name, target_vols)
+        return None, None
+
+    def update_consistencygroup(self, context, group,
+                                add_volumes=None, remove_volumes=None):
+        # Similar to create_consistencygroup, SolidFire's lack of a consistency
+        # group object means there is nothing to update on the cluster.
+        return None, None, None
+
+    def delete_cgsnapshot(self, ctxt, cgsnapshot, snapshots):
+        snap_name = self.configuration.sf_volume_prefix + cgsnapshot['id']
+        self._delete_cgsnapshot_by_name(snap_name)
+        return None, None
+
+    def delete_consistencygroup(self, ctxt, group, volumes):
+        for vol in volumes:
+            self.delete_volume(vol)
+
+        return None, None
+
     def get_volume_stats(self, refresh=False):
         """Get volume status.
 
@@ -1068,6 +1396,8 @@ class SolidFireDriver(san.SanISCSIDriver):
         data["vendor_name"] = 'SolidFire Inc'
         data["driver_version"] = self.VERSION
         data["storage_protocol"] = 'iSCSI'
+        data['consistencygroup_support'] = True
+        data['replication_enabled'] = True
 
         data['total_capacity_gb'] = (
             float(results['maxProvisionedSpace'] / units.Gi))
@@ -1083,6 +1413,17 @@ class SolidFireDriver(san.SanISCSIDriver):
             results['thinProvisioningPercent'])
         self.cluster_stats = data
 
+    def initialize_connection(self, volume, connector, initiator_data=None):
+        """Initialize the connection and return connection info.
+
+           Optionally checks and utilizes volume access groups.
+        """
+        properties = self._sf_initialize_connection(volume,
+                                                    connector,
+                                                    initiator_data)
+        properties['data']['discard'] = True
+        return properties
+
     def attach_volume(self, context, volume,
                       instance_uuid, host_name,
                       mountpoint):
@@ -1107,6 +1448,11 @@ class SolidFireDriver(san.SanISCSIDriver):
 
         self._issue_api_request('ModifyVolume', params)
 
+    def terminate_connection(self, volume, properties, force):
+        return self._sf_terminate_connection(volume,
+                                             properties,
+                                             force)
+
     def detach_volume(self, context, volume, attachment=None):
         sfaccount = self._get_sfaccount(volume['project_id'])
         params = {'accountID': sfaccount['accountID']}
@@ -1186,9 +1532,6 @@ class SolidFireDriver(san.SanISCSIDriver):
 
         if qos:
             params['qos'] = qos
-            for k, v in qos.items():
-                attributes[k] = str(v)
-            params['attributes'] = attributes
 
         self._issue_api_request('ModifyVolume', params)
         return True
@@ -1216,24 +1559,13 @@ class SolidFireDriver(san.SanISCSIDriver):
         sfaccount = self._create_sfaccount(volume['project_id'])
 
         attributes = {}
-        qos = {}
-        if (self.configuration.sf_allow_tenant_qos and
-                volume.get('volume_metadata')is not None):
-            qos = self._set_qos_presets(volume)
-
-        ctxt = context.get_admin_context()
-        type_id = volume.get('volume_type_id', None)
-        if type_id is not None:
-            qos = self._set_qos_by_volume_type(ctxt, type_id)
+        qos = self._retrieve_qos_setting(volume)
 
         import_time = volume['created_at'].isoformat()
         attributes = {'uuid': volume['id'],
                       'is_clone': 'False',
                       'os_imported_at': import_time,
                       'old_name': sfname}
-        if qos:
-            for k, v in qos.items():
-                attributes[k] = str(v)
 
         params = {'name': volume['name'],
                   'volumeID': sf_ref['volumeID'],
@@ -1290,42 +1622,17 @@ class SolidFireDriver(san.SanISCSIDriver):
         self._issue_api_request('ModifyVolume',
                                 params, version='5.0')
 
-    # #### Interface methods for transport layer #### #
-
-    # TODO(jdg): SolidFire can mix and do iSCSI and FC on the
-    # same cluster, we'll modify these later to check based on
-    # the volume info if we need an FC target driver or an
-    # iSCSI target driver
-    def ensure_export(self, context, volume):
-        return self.target_driver.ensure_export(context, volume, None)
-
-    def create_export(self, context, volume, connector):
-        return self.target_driver.create_export(
-            context,
-            volume,
-            None)
-
-    def remove_export(self, context, volume):
-        return self.target_driver.remove_export(context, volume)
-
-    def initialize_connection(self, volume, connector):
-        return self.target_driver.initialize_connection(volume, connector)
-
-    def validate_connector(self, connector):
-        return self.target_driver.validate_connector(connector)
-
-    def terminate_connection(self, volume, connector, **kwargs):
-        return self.target_driver.terminate_connection(volume, connector,
-                                                       **kwargs)
-
 
 class SolidFireISCSI(iscsi_driver.SanISCSITarget):
     def __init__(self, *args, **kwargs):
         super(SolidFireISCSI, self).__init__(*args, **kwargs)
         self.sf_driver = kwargs.get('solidfire_driver')
 
+    def __getattr__(self, attr):
+        return getattr(self.sf_driver, attr)
+
     def _do_iscsi_export(self, volume):
-        sfaccount = self.sf_driver._get_sfaccount(volume['project_id'])
+        sfaccount = self._get_sfaccount(volume['project_id'])
         model_update = {}
         model_update['provider_auth'] = ('CHAP %s %s'
                                          % (sfaccount['username'],
@@ -1350,3 +1657,50 @@ class SolidFireISCSI(iscsi_driver.SanISCSITarget):
 
     def terminate_connection(self, volume, connector, **kwargs):
         pass
+
+    def _sf_initialize_connection(self, volume, connector,
+                                  initiator_data=None):
+        """Initialize the connection and return connection info.
+
+           Optionally checks and utilizes volume access groups.
+        """
+        if self.configuration.sf_enable_vag:
+            iqn = connector['initiator']
+            provider_id = volume['provider_id']
+            vol_id = int(self._parse_provider_id_string(provider_id)[0])
+
+            # safe_create_vag may opt to reuse vs create a vag, so we need to
+            # add our vol_id.
+            vag_id = self._safe_create_vag(iqn, vol_id)
+            self._add_volume_to_vag(vol_id, iqn, vag_id)
+
+        # Continue along with default behavior
+        return super(SolidFireISCSI, self).initialize_connection(volume,
+                                                                 connector)
+
+    def _sf_terminate_connection(self, volume, properties, force):
+        """Terminate the volume connection.
+
+           Optionally remove volume from volume access group.
+           If the VAG is empty then the VAG is also removed.
+        """
+        if self.configuration.sf_enable_vag:
+            iqn = properties['initiator']
+            vag = self._get_vags_by_name(iqn)
+            provider_id = volume['provider_id']
+            vol_id = int(self._parse_provider_id_string(provider_id)[0])
+
+            if vag and not volume['multiattach']:
+                # Multiattach causes problems with removing volumes from VAGs.
+                # Compromise solution for now is to remove multiattach volumes
+                # from VAGs during volume deletion.
+                vag = vag[0]
+                vag_id = vag['volumeAccessGroupID']
+                if [vol_id] == vag['volumes']:
+                    self._remove_vag(vag_id)
+                elif vol_id in vag['volumes']:
+                    self._remove_volume_from_vag(vol_id, vag_id)
+
+        return super(SolidFireISCSI, self).terminate_connection(volume,
+                                                                properties,
+                                                                force=force)
