@@ -13,12 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import base64
-import string
-
 from lxml import etree
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import base64
 from oslo_service import loopingcall
 from six.moves import urllib
 
@@ -65,7 +63,7 @@ def RaiseXIODriverException():
 
 class XIOISEDriver(object):
 
-    VERSION = '1.1.3'
+    VERSION = '1.1.4'
 
     # Version   Changes
     # 1.0.0     Base driver
@@ -73,6 +71,7 @@ class XIOISEDriver(object):
     # 1.1.1     Fix retry loop (Bug 1429283)
     # 1.1.2     Fix host object deletion (Bug 1433450).
     # 1.1.3     Wait for volume/snapshot to be deleted.
+    # 1.1.4     Force target_lun to be int (Bug 1549048)
 
     def __init__(self, *args, **kwargs):
         super(XIOISEDriver, self).__init__()
@@ -150,7 +149,7 @@ class XIOISEDriver(object):
                 support['thin-clones'] = True
         # Make sure ISE support necessary features
         if not support['clones']:
-            LOG.error(_LE("ISE FW version is not compatible with Openstack!"))
+            LOG.error(_LE("ISE FW version is not compatible with OpenStack!"))
             RaiseXIODriverException()
         # set up thin provisioning support
         self.configuration.san_thin_provision = support['thin-clones']
@@ -276,7 +275,7 @@ class XIOISEDriver(object):
             if 'content' in resp:
                 reason = etree.fromstring(resp['content'])
                 if reason is not None:
-                    reason = string.upper(reason.text)
+                    reason = reason.text.upper()
             if INVALID_STATUS in reason:
                 # Request failed with an invalid state. This can be because
                 # source volume is in a temporary unavailable state.
@@ -366,10 +365,10 @@ class XIOISEDriver(object):
         url += uri
         # set up headers for XML and Auth
         header = {'Content-Type': 'application/xml; charset=utf-8'}
-        auth_key =\
-            base64.encodestring('%s:%s' %
-                                (self.configuration.san_login,
-                                 self.configuration.san_password))[:-1]
+        auth_key = ('%s:%s'
+                    % (self.configuration.san_login,
+                       self.configuration.san_password))
+        auth_key = base64.encode_as_text(auth_key)
         header['Authorization'] = 'Basic %s' % auth_key
         # We allow 5 retries on each IP address. If connection to primary
         # fails, secondary will be tried. If connection to secondary is
@@ -412,7 +411,7 @@ class XIOISEDriver(object):
                               primary_ip)
                     RaiseXIODriverException()
                 # swap primary for secondary ip in URL
-                url = string.replace(url, primary_ip, secondary_ip)
+                url = url.replace(primary_ip, secondary_ip)
                 LOG.debug('Trying secondary IP URL: %s', url)
                 using_secondary = 1
                 continue
@@ -605,13 +604,13 @@ class XIOISEDriver(object):
             return vol_info
         # Fill in value and string from status tag attributes.
         vol_info['value'] = status.attrib['value']
-        vol_info['string'] = string.upper(status.attrib['string'])
+        vol_info['string'] = status.attrib['string'].upper()
         # Detailed status has it's own list of tags.
         details = status.find('details')
         if details is not None:
             detail = details.find('detail')
             if detail is not None:
-                vol_info['details'] = string.upper(detail.text)
+                vol_info['details'] = detail.text.upper()
         # Get volume size
         size_tag = volume_node.find('size')
         if size_tag is not None:
@@ -657,7 +656,7 @@ class XIOISEDriver(object):
                     hname_tag = endpoint.find('hostname')
                     if hname_tag is None:
                         continue
-                    if string.upper(hname_tag.text) != string.upper(hostname):
+                    if hname_tag.text.upper() != hostname.upper():
                         continue
                 # Found hostname match. Location string is an attribute in
                 # allocation tag.
@@ -676,7 +675,6 @@ class XIOISEDriver(object):
         # Set up params with volume name, host name and target lun, if
         # specified.
         target_lun = lun
-        params = {}
         params = {'volumename': volume['name'],
                   'hostname': hostname}
         # Fill in LUN if specified.
@@ -769,7 +767,7 @@ class XIOISEDriver(object):
                 gid = endpoint_node.find('globalid')
                 if gid is None:
                     continue
-                if string.upper(gid.text) != string.upper(ep):
+                if gid.text.upper() != ep.upper():
                     continue
                 # We have a match. Fill in host name, type and locator
                 host['locator'] = host_node.attrib['self']
@@ -802,8 +800,7 @@ class XIOISEDriver(object):
         # Log host creation.
         LOG.debug("Create host %(host)s; %(endpoint)s",
                   {'host': hostname, 'endpoint': endpoint_str})
-        # Issue REST call to create host entry of Openstack type.
-        params = {}
+        # Issue REST call to create host entry of OpenStack type.
         params = {'name': hostname, 'endpoint': endpoint_str,
                   'os': 'openstack'}
         url = '/storage/arrays/%s/hosts' % (self._get_ise_globalid())
@@ -1018,8 +1015,7 @@ class XIOISEDriver(object):
             # count volumes
             volumes = child.find('volumes')
             if volumes is not None:
-                for volume in volumes:
-                    vol_cnt += 1
+                vol_cnt += len(volumes)
         return (pools, vol_cnt)
 
     def _update_volume_stats(self):
@@ -1081,20 +1077,20 @@ class XIOISEDriver(object):
                     fields = key.split(':')
                     key = fields[0]
                     subkey = fields[1]
-                if string.upper(key) == string.upper('Feature'):
-                    if string.upper(subkey) == string.upper('Raid'):
+                if key.upper() == 'Feature'.upper():
+                    if subkey.upper() == 'Raid'.upper():
                         specs['raid'] = value
-                    elif string.upper(subkey) == string.upper('Pool'):
+                    elif subkey.upper() == 'Pool'.upper():
                         specs['pool'] = value
-                elif string.upper(key) == string.upper('Affinity'):
+                elif key.upper() == 'Affinity'.upper():
                     # Only fill this in if ISE FW supports volume affinity
                     if self.configuration.ise_affinity:
-                        if string.upper(subkey) == string.upper('Type'):
+                        if subkey.upper() == 'Type'.upper():
                             specs['affinity'] = value
-                elif string.upper(key) == string.upper('Alloc'):
+                elif key.upper() == 'Alloc'.upper():
                     # Only fill this in if ISE FW supports thin provisioning
                     if self.configuration.san_thin_provision:
-                        if string.upper(subkey) == string.upper('Type'):
+                        if subkey.upper() == 'Type'.upper():
                             specs['alloctype'] = value
         return specs
 
@@ -1116,11 +1112,11 @@ class XIOISEDriver(object):
                 if ':' in key:
                     fields = key.split(':')
                     key = fields[1]
-                if string.upper(key) == string.upper('minIOPS'):
+                if key.upper() == 'minIOPS'.upper():
                     specs['minIOPS'] = value
-                elif string.upper(key) == string.upper('maxIOPS'):
+                elif key.upper() == 'maxIOPS'.upper():
                     specs['maxIOPS'] = value
-                elif string.upper(key) == string.upper('burstIOPS'):
+                elif key.upper() == 'burstIOPS'.upper():
                     specs['burstIOPS'] = value
         return specs
 
@@ -1334,8 +1330,8 @@ class XIOISEDriver(object):
                 # host still not found, this is fatal.
                 LOG.error(_LE("Host could not be found!"))
                 RaiseXIODriverException()
-        elif string.upper(host['type']) != 'OPENSTACK':
-            # Make sure host type is marked as Openstack host
+        elif host['type'].upper() != 'OPENSTACK':
+            # Make sure host type is marked as OpenStack host
             params = {'os': 'openstack'}
             resp = self._send_cmd('PUT', host['locator'], params)
             status = resp['status']
@@ -1348,9 +1344,8 @@ class XIOISEDriver(object):
         target_lun = self._present_volume(volume, host['name'], target_lun)
         # Fill in target information.
         data = {}
-        data['target_lun'] = target_lun
+        data['target_lun'] = int(target_lun)
         data['volume_id'] = volume['id']
-        data['access_mode'] = 'rw'
         return data
 
     def ise_unpresent(self, volume, endpoints):
@@ -1421,7 +1416,6 @@ class XIOISEISCSIDriver(driver.ISCSIDriver):
         self.driver.create_volume(volume)
         # Volume created successfully. Fill in CHAP information.
         model_update = {}
-        chap = {}
         chap = self.driver.find_target_chap()
         if chap['chap_user'] != '':
             model_update['provider_auth'] = 'CHAP %s %s' % \
