@@ -1,8 +1,6 @@
 #    (c) Copyright 2014 Brocade Communications Systems Inc.
 #    All Rights Reserved.
 #
-#    Copyright 2014 OpenStack Foundation
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -34,7 +32,7 @@ from cinder import exception
 from cinder.i18n import _, _LE
 from cinder import ssh_utils
 from cinder import utils
-import cinder.zonemanager.drivers.brocade.fc_zone_constants as ZoneConstant
+import cinder.zonemanager.drivers.brocade.fc_zone_constants as zone_constant
 
 LOG = logging.getLogger(__name__)
 
@@ -44,14 +42,17 @@ class BrcdFCZoneClientCLI(object):
     switch_port = '22'
     switch_user = 'admin'
     switch_pwd = 'none'
+    switch_key = 'none'
     patrn = re.compile('[;\s]+')
 
-    def __init__(self, ipaddress, username, password, port):
-        """initializing the client."""
+    def __init__(self, ipaddress, username,
+                 password, port, key):
+        """Initializing the client."""
         self.switch_ip = ipaddress
         self.switch_port = port
         self.switch_user = username
         self.switch_pwd = password
+        self.switch_key = key
         self.sshpool = None
 
     def get_active_zone_set(self):
@@ -77,7 +78,7 @@ class BrcdFCZoneClientCLI(object):
         zone_set_name = None
         try:
             switch_data = self._get_switch_info(
-                [ZoneConstant.GET_ACTIVE_ZONE_CFG])
+                [zone_constant.GET_ACTIVE_ZONE_CFG])
         except exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
                 LOG.error(_LE("Failed getting active zone set "
@@ -91,7 +92,7 @@ class BrcdFCZoneClientCLI(object):
                     line_split = [x.replace(
                         ' ',
                         '') for x in line_split]
-                    if ZoneConstant.CFG_ZONESET in line_split:
+                    if zone_constant.CFG_ZONESET in line_split:
                         zone_set_name = line_split[1]
                         continue
                     if line_split[1]:
@@ -101,10 +102,10 @@ class BrcdFCZoneClientCLI(object):
                         zone_member = line_split[2]
                         zone_member_list = zone.get(zone_name)
                         zone_member_list.append(zone_member)
-            zone_set[ZoneConstant.CFG_ZONES] = zone
-            zone_set[ZoneConstant.ACTIVE_ZONE_CONFIG] = zone_set_name
+            zone_set[zone_constant.CFG_ZONES] = zone
+            zone_set[zone_constant.ACTIVE_ZONE_CONFIG] = zone_set_name
         except Exception:
-            # Incase of parsing error here, it should be malformed cli output.
+            # In case of parsing error here, it should be malformed cli output.
             msg = _("Malformed zone configuration: (switch=%(switch)s "
                     "zone_config=%(zone_config)s)."
                     ) % {'switch': self.switch_ip,
@@ -137,12 +138,16 @@ class BrcdFCZoneClientCLI(object):
         if not active_zone_set:
             active_zone_set = self.get_active_zone_set()
             LOG.debug("Active zone set: %s", active_zone_set)
-        zone_list = active_zone_set[ZoneConstant.CFG_ZONES]
+        zone_list = active_zone_set[zone_constant.CFG_ZONES]
         LOG.debug("zone list: %s", zone_list)
         for zone in zones.keys():
-            # if zone exists, its an update. Delete & insert
-            # TODO(skolathur): This can be optimized to an update call later
+            # If zone exists, its an update. Delete & insert
+            # TODO(skolathur): This still need to be optimized
+            # to an update call later. Now we just handled the
+            # same zone name with same zone members.
             if (zone in zone_list):
+                if set(zones[zone]) == set(zone_list[zone]):
+                    break
                 try:
                     self.delete_zones(zone, activate, active_zone_set)
                 except exception.BrocadeZoningCliException:
@@ -162,14 +167,16 @@ class BrcdFCZoneClientCLI(object):
                 zone_with_sep += ';'
             iterator_count += 1
             zone_with_sep += zone
+        if not zone_with_sep:
+            return
         try:
             # Get active zone set from device, as some of the zones
             # could be deleted.
             active_zone_set = self.get_active_zone_set()
-            cfg_name = active_zone_set[ZoneConstant.ACTIVE_ZONE_CONFIG]
+            cfg_name = active_zone_set[zone_constant.ACTIVE_ZONE_CONFIG]
             cmd = None
             if not cfg_name:
-                cfg_name = ZoneConstant.OPENSTACK_CFG_NAME
+                cfg_name = zone_constant.OPENSTACK_CFG_NAME
                 cmd = 'cfgcreate "%(zoneset)s", "%(zones)s"' \
                     % {'zoneset': cfg_name, 'zones': zone_with_sep}
             else:
@@ -191,21 +198,21 @@ class BrcdFCZoneClientCLI(object):
 
     def activate_zoneset(self, cfgname):
         """Method to Activate the zone config. Param cfgname - ZonesetName."""
-        cmd_list = [ZoneConstant.ACTIVATE_ZONESET, cfgname]
+        cmd_list = [zone_constant.ACTIVATE_ZONESET, cfgname]
         return self._ssh_execute(cmd_list, True, 1)
 
     def deactivate_zoneset(self):
         """Method to deActivate the zone config."""
-        return self._ssh_execute([ZoneConstant.DEACTIVATE_ZONESET], True, 1)
+        return self._ssh_execute([zone_constant.DEACTIVATE_ZONESET], True, 1)
 
     def delete_zones(self, zone_names, activate, active_zone_set=None):
         """Delete zones from fabric.
 
         Method to delete the active zone config zones
 
-        params zone_names: zoneNames separated by semicolon
-        params activate: True/False
-        params active_zone_set: the active zone set dict retrieved
+        :param zone_names: zoneNames separated by semicolon
+        :param activate: True/False
+        :param active_zone_set: the active zone set dict retrieved
                                 from get_active_zone_set method
         """
         active_zoneset_name = None
@@ -213,8 +220,8 @@ class BrcdFCZoneClientCLI(object):
         if not active_zone_set:
             active_zone_set = self.get_active_zone_set()
         active_zoneset_name = active_zone_set[
-            ZoneConstant.ACTIVE_ZONE_CONFIG]
-        zone_list = active_zone_set[ZoneConstant.CFG_ZONES]
+            zone_constant.ACTIVE_ZONE_CONFIG]
+        zone_list = active_zone_set[zone_constant.CFG_ZONES]
         zones = self.patrn.split(''.join(zone_names))
         cmd = None
         try:
@@ -254,8 +261,8 @@ class BrcdFCZoneClientCLI(object):
         return_list = []
         try:
             cmd = '%(nsshow)s;%(nscamshow)s' % {
-                'nsshow': ZoneConstant.NS_SHOW,
-                'nscamshow': ZoneConstant.NS_CAM_SHOW}
+                'nsshow': zone_constant.NS_SHOW,
+                'nscamshow': zone_constant.NS_CAM_SHOW}
             cli_output = self._get_switch_info([cmd])
         except exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
@@ -267,7 +274,7 @@ class BrcdFCZoneClientCLI(object):
         return return_list
 
     def _cfg_save(self):
-        self._ssh_execute([ZoneConstant.CFG_SAVE], True, 1)
+        self._ssh_execute([zone_constant.CFG_SAVE], True, 1)
 
     def _zone_delete(self, zone_name):
         cmd = 'zonedelete "%(zone_name)s"' % {'zone_name': zone_name}
@@ -276,17 +283,17 @@ class BrcdFCZoneClientCLI(object):
     def _cfg_trans_abort(self):
         is_abortable = self._is_trans_abortable()
         if(is_abortable):
-            self.apply_zone_change([ZoneConstant.CFG_ZONE_TRANS_ABORT])
+            self.apply_zone_change([zone_constant.CFG_ZONE_TRANS_ABORT])
 
     def _is_trans_abortable(self):
         is_abortable = False
         stdout, stderr = None, None
         stdout, stderr = self._run_ssh(
-            [ZoneConstant.CFG_SHOW_TRANS], True, 1)
+            [zone_constant.CFG_SHOW_TRANS], True, 1)
         output = stdout.splitlines()
         is_abortable = False
         for line in output:
-            if(ZoneConstant.TRANS_ABORTABLE in line):
+            if(zone_constant.TRANS_ABORTABLE in line):
                 is_abortable = True
                 break
         if stderr:
@@ -386,6 +393,7 @@ class BrcdFCZoneClientCLI(object):
                                              None,
                                              self.switch_user,
                                              self.switch_pwd,
+                                             self.switch_key,
                                              min_size=1,
                                              max_size=5)
         last_exception = None
@@ -432,6 +440,7 @@ class BrcdFCZoneClientCLI(object):
                                              None,
                                              self.switch_user,
                                              self.switch_pwd,
+                                             self.switch_key,
                                              min_size=1,
                                              max_size=5)
         stdin, stdout, stderr = None, None, None
@@ -443,7 +452,7 @@ class BrcdFCZoneClientCLI(object):
                     attempts -= 1
                     try:
                         stdin, stdout, stderr = ssh.exec_command(command)
-                        stdin.write("%s\n" % ZoneConstant.YES)
+                        stdin.write("%s\n" % zone_constant.YES)
                         channel = stdout.channel
                         exit_status = channel.recv_exit_status()
                         LOG.debug("Exit Status from ssh: %s", exit_status)
@@ -493,7 +502,7 @@ class BrcdFCZoneClientCLI(object):
     def _execute_shell_cmd(self, cmd):
         """Run command over shell for older firmware versions.
 
-        We invoke shell and issue the command and return the output.
+        Invokes shell and issue the command and return the output.
         This is primarily used for issuing read commands when we are not sure
         if the firmware supports exec_command.
         """
@@ -506,6 +515,7 @@ class BrcdFCZoneClientCLI(object):
                                              None,
                                              self.switch_user,
                                              self.switch_pwd,
+                                             self.switch_key,
                                              min_size=1,
                                              max_size=5)
         with self.sshpool.item() as ssh:
@@ -539,6 +549,7 @@ exit
                 channel.close()
             except Exception:
                 LOG.exception(_LE('Error closing channel.'))
+            LOG.debug("_execute_cmd: stdout to return: %s", stdout)
             LOG.debug("_execute_cmd: stderr to return: %s", stderr)
         return (stdout, stderr)
 
